/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */






    /// https://drafts.csswg.org/css-fonts-3/#propdef-font
    pub mod font {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        pub struct Longhands {
                pub font_style:
                    longhands::font_style::SpecifiedValue
                    ,
                pub font_variant_caps:
                    longhands::font_variant_caps::SpecifiedValue
                    ,
                pub font_weight:
                    longhands::font_weight::SpecifiedValue
                    ,
                pub font_stretch:
                    longhands::font_stretch::SpecifiedValue
                    ,
                pub font_size:
                    longhands::font_size::SpecifiedValue
                    ,
                pub line_height:
                    longhands::line_height::SpecifiedValue
                    ,
                pub font_family:
                    longhands::font_family::SpecifiedValue
                    ,
                pub font_size_adjust:
                    longhands::font_size_adjust::SpecifiedValue
                    ,
                pub font_kerning:
                    longhands::font_kerning::SpecifiedValue
                    ,
                pub font_optical_sizing:
                    longhands::font_optical_sizing::SpecifiedValue
                    ,
                pub font_variant_alternates:
                    longhands::font_variant_alternates::SpecifiedValue
                    ,
                pub font_variant_east_asian:
                    longhands::font_variant_east_asian::SpecifiedValue
                    ,
                pub font_variant_ligatures:
                    longhands::font_variant_ligatures::SpecifiedValue
                    ,
                pub font_variant_numeric:
                    longhands::font_variant_numeric::SpecifiedValue
                    ,
                pub font_variant_position:
                    longhands::font_variant_position::SpecifiedValue
                    ,
                pub font_language_override:
                    longhands::font_language_override::SpecifiedValue
                    ,
                pub font_feature_settings:
                    longhands::font_feature_settings::SpecifiedValue
                    ,
                pub font_variation_settings:
                    longhands::font_variation_settings::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub font_style:
                    &'a longhands::font_style::SpecifiedValue,
                pub font_variant_caps:
                    &'a longhands::font_variant_caps::SpecifiedValue,
                pub font_weight:
                    &'a longhands::font_weight::SpecifiedValue,
                pub font_stretch:
                    &'a longhands::font_stretch::SpecifiedValue,
                pub font_size:
                    &'a longhands::font_size::SpecifiedValue,
                pub line_height:
                    &'a longhands::line_height::SpecifiedValue,
                pub font_family:
                    &'a longhands::font_family::SpecifiedValue,
                pub font_size_adjust:
                    &'a longhands::font_size_adjust::SpecifiedValue,
                pub font_kerning:
                    &'a longhands::font_kerning::SpecifiedValue,
                pub font_optical_sizing:
                    Option<
                    &'a longhands::font_optical_sizing::SpecifiedValue,
                    >,
                pub font_variant_alternates:
                    &'a longhands::font_variant_alternates::SpecifiedValue,
                pub font_variant_east_asian:
                    &'a longhands::font_variant_east_asian::SpecifiedValue,
                pub font_variant_ligatures:
                    &'a longhands::font_variant_ligatures::SpecifiedValue,
                pub font_variant_numeric:
                    &'a longhands::font_variant_numeric::SpecifiedValue,
                pub font_variant_position:
                    &'a longhands::font_variant_position::SpecifiedValue,
                pub font_language_override:
                    &'a longhands::font_language_override::SpecifiedValue,
                pub font_feature_settings:
                    &'a longhands::font_feature_settings::SpecifiedValue,
                pub font_variation_settings:
                    Option<
                    &'a longhands::font_variation_settings::SpecifiedValue,
                    >,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter(iter: impl Iterator<Item = &'a PropertyDeclaration>) -> Result<Self, ()> {
                // Define all of the expected variables that correspond to the shorthand
                    let mut font_style =
                        None::< &'a longhands::font_style::SpecifiedValue>;
                    let mut font_variant_caps =
                        None::< &'a longhands::font_variant_caps::SpecifiedValue>;
                    let mut font_weight =
                        None::< &'a longhands::font_weight::SpecifiedValue>;
                    let mut font_stretch =
                        None::< &'a longhands::font_stretch::SpecifiedValue>;
                    let mut font_size =
                        None::< &'a longhands::font_size::SpecifiedValue>;
                    let mut line_height =
                        None::< &'a longhands::line_height::SpecifiedValue>;
                    let mut font_family =
                        None::< &'a longhands::font_family::SpecifiedValue>;
                    let mut font_size_adjust =
                        None::< &'a longhands::font_size_adjust::SpecifiedValue>;
                    let mut font_kerning =
                        None::< &'a longhands::font_kerning::SpecifiedValue>;
                    let mut font_optical_sizing =
                        None::< &'a longhands::font_optical_sizing::SpecifiedValue>;
                    let mut font_variant_alternates =
                        None::< &'a longhands::font_variant_alternates::SpecifiedValue>;
                    let mut font_variant_east_asian =
                        None::< &'a longhands::font_variant_east_asian::SpecifiedValue>;
                    let mut font_variant_ligatures =
                        None::< &'a longhands::font_variant_ligatures::SpecifiedValue>;
                    let mut font_variant_numeric =
                        None::< &'a longhands::font_variant_numeric::SpecifiedValue>;
                    let mut font_variant_position =
                        None::< &'a longhands::font_variant_position::SpecifiedValue>;
                    let mut font_language_override =
                        None::< &'a longhands::font_language_override::SpecifiedValue>;
                    let mut font_feature_settings =
                        None::< &'a longhands::font_feature_settings::SpecifiedValue>;
                    let mut font_variation_settings =
                        None::< &'a longhands::font_variation_settings::SpecifiedValue>;

                // Attempt to assign the incoming declarations to the expected variables
                for declaration in iter {
                    match *declaration {
                            PropertyDeclaration::FontStyle(ref value) => {
                                font_style = Some(value)
                            },
                            PropertyDeclaration::FontVariantCaps(ref value) => {
                                font_variant_caps = Some(value)
                            },
                            PropertyDeclaration::FontWeight(ref value) => {
                                font_weight = Some(value)
                            },
                            PropertyDeclaration::FontStretch(ref value) => {
                                font_stretch = Some(value)
                            },
                            PropertyDeclaration::FontSize(ref value) => {
                                font_size = Some(value)
                            },
                            PropertyDeclaration::LineHeight(ref value) => {
                                line_height = Some(value)
                            },
                            PropertyDeclaration::FontFamily(ref value) => {
                                font_family = Some(value)
                            },
                            PropertyDeclaration::FontSizeAdjust(ref value) => {
                                font_size_adjust = Some(value)
                            },
                            PropertyDeclaration::FontKerning(ref value) => {
                                font_kerning = Some(value)
                            },
                            PropertyDeclaration::FontOpticalSizing(ref value) => {
                                font_optical_sizing = Some(value)
                            },
                            PropertyDeclaration::FontVariantAlternates(ref value) => {
                                font_variant_alternates = Some(value)
                            },
                            PropertyDeclaration::FontVariantEastAsian(ref value) => {
                                font_variant_east_asian = Some(value)
                            },
                            PropertyDeclaration::FontVariantLigatures(ref value) => {
                                font_variant_ligatures = Some(value)
                            },
                            PropertyDeclaration::FontVariantNumeric(ref value) => {
                                font_variant_numeric = Some(value)
                            },
                            PropertyDeclaration::FontVariantPosition(ref value) => {
                                font_variant_position = Some(value)
                            },
                            PropertyDeclaration::FontLanguageOverride(ref value) => {
                                font_language_override = Some(value)
                            },
                            PropertyDeclaration::FontFeatureSettings(ref value) => {
                                font_feature_settings = Some(value)
                            },
                            PropertyDeclaration::FontVariationSettings(ref value) => {
                                font_variation_settings = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        font_style,
                        font_variant_caps,
                        font_weight,
                        font_stretch,
                        font_size,
                        line_height,
                        font_family,
                        font_size_adjust,
                        font_kerning,
                        font_optical_sizing,
                        font_variant_alternates,
                        font_variant_east_asian,
                        font_variant_ligatures,
                        font_variant_numeric,
                        font_variant_position,
                        font_language_override,
                        font_feature_settings,
                        font_variation_settings,
                ) {

                    (
                        Some(font_style),
                        Some(font_variant_caps),
                        Some(font_weight),
                        Some(font_stretch),
                        Some(font_size),
                        Some(line_height),
                        Some(font_family),
                        Some(font_size_adjust),
                        Some(font_kerning),
                        font_optical_sizing,
                        Some(font_variant_alternates),
                        Some(font_variant_east_asian),
                        Some(font_variant_ligatures),
                        Some(font_variant_numeric),
                        Some(font_variant_position),
                        Some(font_language_override),
                        Some(font_feature_settings),
                        font_variation_settings,
                    ) =>
                    Ok(LonghandsToSerialize {
                            font_style,
                            font_variant_caps,
                            font_weight,
                            font_stretch,
                            font_size,
                            line_height,
                            font_family,
                            font_size_adjust,
                            font_kerning,
                            font_optical_sizing,
                            font_variant_alternates,
                            font_variant_east_asian,
                            font_variant_ligatures,
                            font_variant_numeric,
                            font_variant_position,
                            font_language_override,
                            font_feature_settings,
                            font_variation_settings,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::FontStyle(
                        longhands.font_style
                    ));
                    declarations.push(PropertyDeclaration::FontVariantCaps(
                        longhands.font_variant_caps
                    ));
                    declarations.push(PropertyDeclaration::FontWeight(
                        longhands.font_weight
                    ));
                    declarations.push(PropertyDeclaration::FontStretch(
                        longhands.font_stretch
                    ));
                    declarations.push(PropertyDeclaration::FontSize(
                        longhands.font_size
                    ));
                    declarations.push(PropertyDeclaration::LineHeight(
                        longhands.line_height
                    ));
                    declarations.push(PropertyDeclaration::FontFamily(
                        longhands.font_family
                    ));
                    declarations.push(PropertyDeclaration::FontSizeAdjust(
                        longhands.font_size_adjust
                    ));
                    declarations.push(PropertyDeclaration::FontKerning(
                        longhands.font_kerning
                    ));
                if NonCustomPropertyId::from(LonghandId::FontOpticalSizing)
                    .allowed_in_ignoring_rule_type(context) {
                    declarations.push(PropertyDeclaration::FontOpticalSizing(
                        longhands.font_optical_sizing
                    ));
                }
                    declarations.push(PropertyDeclaration::FontVariantAlternates(
                        longhands.font_variant_alternates
                    ));
                    declarations.push(PropertyDeclaration::FontVariantEastAsian(
                        longhands.font_variant_east_asian
                    ));
                    declarations.push(PropertyDeclaration::FontVariantLigatures(
                        longhands.font_variant_ligatures
                    ));
                    declarations.push(PropertyDeclaration::FontVariantNumeric(
                        longhands.font_variant_numeric
                    ));
                    declarations.push(PropertyDeclaration::FontVariantPosition(
                        longhands.font_variant_position
                    ));
                    declarations.push(PropertyDeclaration::FontLanguageOverride(
                        longhands.font_language_override
                    ));
                    declarations.push(PropertyDeclaration::FontFeatureSettings(
                        longhands.font_feature_settings
                    ));
                if NonCustomPropertyId::from(LonghandId::FontVariationSettings)
                    .allowed_in_ignoring_rule_type(context) {
                    declarations.push(PropertyDeclaration::FontVariationSettings(
                        longhands.font_variation_settings
                    ));
                }
            })
        }

        /// Try to serialize a given shorthand to a string.
        pub fn to_css(declarations: &[&PropertyDeclaration], dest: &mut crate::str::CssStringWriter) -> fmt::Result {
            match LonghandsToSerialize::from_iter(declarations.iter().cloned()) {
                Ok(longhands) => longhands.to_css(&mut CssWriter::new(dest)),
                Err(_) => Ok(())
            }
        }

        
    use crate::parser::Parse;
    use crate::properties::longhands::{font_family, font_style, font_weight, font_stretch};
    use crate::properties::longhands::font_variant_caps;
    use crate::values::specified::text::LineHeight;
    use crate::values::specified::FontSize;
    use crate::values::specified::font::{FontStretch, FontStretchKeyword};
    #[cfg(feature = "gecko")]
    use crate::values::specified::font::SystemFont;

    
            use crate::properties::longhands::font_kerning;
            use crate::properties::longhands::font_language_override;
            use crate::properties::longhands::font_size_adjust;
            use crate::properties::longhands::font_variant_alternates;
            use crate::properties::longhands::font_variant_east_asian;
            use crate::properties::longhands::font_variant_ligatures;
            use crate::properties::longhands::font_variant_numeric;
            use crate::properties::longhands::font_variant_position;
            use crate::properties::longhands::font_feature_settings;
            use crate::properties::longhands::font_variation_settings;
            use crate::properties::longhands::font_optical_sizing;
    use self::font_family::SpecifiedValue as FontFamily;

    pub fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let mut nb_normals = 0;
        let mut style = None;
        let mut variant_caps = None;
        let mut weight = None;
        let mut stretch = None;
        let size;
            if let Ok(sys) = input.try_parse(SystemFont::parse) {
                return Ok(expanded! {
                             font_family: font_family::SpecifiedValue::system_font(sys),
                             font_size: FontSize::system_font(sys),
                             font_style: font_style::SpecifiedValue::system_font(sys),
                             font_variant_caps: font_variant_caps::SpecifiedValue::system_font(sys),
                             font_stretch: font_stretch::SpecifiedValue::system_font(sys),
                             font_kerning: font_kerning::SpecifiedValue::system_font(sys),
                             font_variant_position: font_variant_position::SpecifiedValue::system_font(sys),
                             font_weight: font_weight::SpecifiedValue::system_font(sys),
                             font_size_adjust: font_size_adjust::SpecifiedValue::system_font(sys),
                             font_variant_alternates: font_variant_alternates::SpecifiedValue::system_font(sys),
                             font_variant_ligatures: font_variant_ligatures::SpecifiedValue::system_font(sys),
                             font_variant_east_asian: font_variant_east_asian::SpecifiedValue::system_font(sys),
                             font_variant_numeric: font_variant_numeric::SpecifiedValue::system_font(sys),
                             font_language_override: font_language_override::SpecifiedValue::system_font(sys),
                             font_feature_settings: font_feature_settings::SpecifiedValue::system_font(sys),
                             font_variation_settings: font_variation_settings::SpecifiedValue::system_font(sys),
                             font_optical_sizing: font_optical_sizing::SpecifiedValue::system_font(sys),
                     // line-height is just reset to initial
                     line_height: LineHeight::normal(),
                 })
            }
        loop {
            // Special-case 'normal' because it is valid in each of
            // font-style, font-weight, font-variant and font-stretch.
            // Leaves the values to None, 'normal' is the initial value for each of them.
            if input.try_parse(|input| input.expect_ident_matching("normal")).is_ok() {
                nb_normals += 1;
                continue;
            }
            if style.is_none() {
                if let Ok(value) = input.try_parse(|input| font_style::parse(context, input)) {
                    style = Some(value);
                    continue
                }
            }
            if weight.is_none() {
                if let Ok(value) = input.try_parse(|input| font_weight::parse(context, input)) {
                    weight = Some(value);
                    continue
                }
            }
            if variant_caps.is_none() {
                if let Ok(value) = input.try_parse(|input| font_variant_caps::parse(context, input)) {
                    variant_caps = Some(value);
                    continue
                }
            }
            if stretch.is_none() {
                if let Ok(value) = input.try_parse(FontStretchKeyword::parse) {
                    stretch = Some(FontStretch::Keyword(value));
                    continue
                }
            }
            size = Some(FontSize::parse(context, input)?);
            break
        }

        let size = match size {
            Some(s) => s,
            None => {
                return Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError))
            }
        };

        let line_height = if input.try_parse(|input| input.expect_delim('/')).is_ok() {
            Some(LineHeight::parse(context, input)?)
        } else {
            None
        };

        #[inline]
        fn count<T>(opt: &Option<T>) -> u8 {
            if opt.is_some() { 1 } else { 0 }
        }

        if (count(&style) + count(&weight) + count(&variant_caps) + count(&stretch) + nb_normals) > 4 {
            return Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError))
        }

        let family = FontFamily::parse(context, input)?;
        Ok(expanded! {
                font_style: unwrap_or_initial!(font_style, style),
                font_weight: unwrap_or_initial!(font_weight, weight),
                font_stretch: unwrap_or_initial!(font_stretch, stretch),
                font_variant_caps: unwrap_or_initial!(font_variant_caps, variant_caps),
            font_size: size,
            line_height: line_height.unwrap_or(LineHeight::normal()),
            font_family: family,
                    font_kerning: font_kerning::get_initial_specified_value(),
                    font_language_override: font_language_override::get_initial_specified_value(),
                    font_size_adjust: font_size_adjust::get_initial_specified_value(),
                    font_variant_alternates: font_variant_alternates::get_initial_specified_value(),
                    font_variant_east_asian: font_variant_east_asian::get_initial_specified_value(),
                    font_variant_ligatures: font_variant_ligatures::get_initial_specified_value(),
                    font_variant_numeric: font_variant_numeric::get_initial_specified_value(),
                    font_variant_position: font_variant_position::get_initial_specified_value(),
                    font_feature_settings: font_feature_settings::get_initial_specified_value(),
                    font_variation_settings: font_variation_settings::get_initial_specified_value(),
                    font_optical_sizing: font_optical_sizing::get_initial_specified_value(),
        })
    }

        enum CheckSystemResult {
            AllSystem(SystemFont),
            SomeSystem,
            None
        }

    impl<'a> ToCss for LonghandsToSerialize<'a> {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
                match self.check_system() {
                    CheckSystemResult::AllSystem(sys) => return sys.to_css(dest),
                    CheckSystemResult::SomeSystem => return Ok(()),
                    CheckSystemResult::None => {}
                }

            if let Some(v) = self.font_optical_sizing {
                if v != &font_optical_sizing::get_initial_specified_value() {
                    return Ok(());
                }
            }
            if let Some(v) = self.font_variation_settings {
                if v != &font_variation_settings::get_initial_specified_value() {
                    return Ok(());
                }
            }

            if self.font_kerning != &font_kerning::get_initial_specified_value() {
                return Ok(());
            }
            if self.font_language_override != &font_language_override::get_initial_specified_value() {
                return Ok(());
            }
            if self.font_size_adjust != &font_size_adjust::get_initial_specified_value() {
                return Ok(());
            }
            if self.font_variant_alternates != &font_variant_alternates::get_initial_specified_value() {
                return Ok(());
            }
            if self.font_variant_east_asian != &font_variant_east_asian::get_initial_specified_value() {
                return Ok(());
            }
            if self.font_variant_ligatures != &font_variant_ligatures::get_initial_specified_value() {
                return Ok(());
            }
            if self.font_variant_numeric != &font_variant_numeric::get_initial_specified_value() {
                return Ok(());
            }
            if self.font_variant_position != &font_variant_position::get_initial_specified_value() {
                return Ok(());
            }
            if self.font_feature_settings != &font_feature_settings::get_initial_specified_value() {
                return Ok(());
            }

            // Only font-stretch keywords are allowed as part as the font
            // shorthand.
            let font_stretch = match *self.font_stretch {
                FontStretch::Keyword(kw) => kw,
                FontStretch::Stretch(percentage) => {
                    match FontStretchKeyword::from_percentage(percentage.0.get()) {
                        Some(kw) => kw,
                        None => return Ok(()),
                    }
                }
                FontStretch::System(..) => return Ok(()),
            };

                if self.font_style != &font_style::get_initial_specified_value() {
                    self.font_style.to_css(dest)?;
                    dest.write_str(" ")?;
                }
                if self.font_variant_caps != &font_variant_caps::get_initial_specified_value() {
                    self.font_variant_caps.to_css(dest)?;
                    dest.write_str(" ")?;
                }
                if self.font_weight != &font_weight::get_initial_specified_value() {
                    self.font_weight.to_css(dest)?;
                    dest.write_str(" ")?;
                }

            if font_stretch != FontStretchKeyword::Normal {
                font_stretch.to_css(dest)?;
                dest.write_str(" ")?;
            }

            self.font_size.to_css(dest)?;

            if *self.line_height != LineHeight::normal() {
                dest.write_str(" / ")?;
                self.line_height.to_css(dest)?;
            }

            dest.write_str(" ")?;
            self.font_family.to_css(dest)?;

            Ok(())
        }
    }

    impl<'a> LonghandsToSerialize<'a> {
        /// Check if some or all members are system fonts
        fn check_system(&self) -> CheckSystemResult {
            let mut sys = None;
            let mut all = true;

            {
                let value = self.font_family;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_size;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_style;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_variant_caps;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_stretch;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_kerning;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_variant_position;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_weight;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_size_adjust;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_variant_alternates;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_variant_ligatures;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_variant_east_asian;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_variant_numeric;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_language_override;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            {
                let value = self.font_feature_settings;
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            if let Some(value) = self.font_variation_settings {
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            if let Some(value) = self.font_optical_sizing {
                match value.get_system() {
                    Some(s) => {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    }
                    None => {
                        all = false;
                    }
                }
            }
            if self.line_height != &LineHeight::normal() {
                all = false
            }
            if all {
                CheckSystemResult::AllSystem(sys.unwrap())
            } else if sys.is_some() {
                CheckSystemResult::SomeSystem
            } else {
                CheckSystemResult::None
            }
        }
    }

    
    impl SpecifiedValueInfo for Longhands {
        const SUPPORTED_TYPES: u8 = 0
            | <longhands::font_style::SpecifiedValue as SpecifiedValueInfo>::SUPPORTED_TYPES
            | <longhands::font_weight::SpecifiedValue as SpecifiedValueInfo>::SUPPORTED_TYPES
            | <longhands::font_stretch::SpecifiedValue as SpecifiedValueInfo>::SUPPORTED_TYPES
            | <longhands::font_variant_caps::SpecifiedValue as SpecifiedValueInfo>::SUPPORTED_TYPES
            | <longhands::font_size::SpecifiedValue as SpecifiedValueInfo>::SUPPORTED_TYPES
            | <longhands::font_family::SpecifiedValue as SpecifiedValueInfo>::SUPPORTED_TYPES
            ;

        fn collect_completion_keywords(f: KeywordsCollectFn) {
            <longhands::font_style::SpecifiedValue as SpecifiedValueInfo>::collect_completion_keywords(f);
            <longhands::font_weight::SpecifiedValue as SpecifiedValueInfo>::collect_completion_keywords(f);
            <longhands::font_stretch::SpecifiedValue as SpecifiedValueInfo>::collect_completion_keywords(f);
            <longhands::font_variant_caps::SpecifiedValue as SpecifiedValueInfo>::collect_completion_keywords(f);
            <longhands::font_size::SpecifiedValue as SpecifiedValueInfo>::collect_completion_keywords(f);
            <longhands::font_family::SpecifiedValue as SpecifiedValueInfo>::collect_completion_keywords(f);
            <SystemFont as SpecifiedValueInfo>::collect_completion_keywords(f);
        }
    }

    }




    /// https://drafts.csswg.org/css-fonts-3/#propdef-font-variant
    pub mod font_variant {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub font_variant_caps:
                    longhands::font_variant_caps::SpecifiedValue
                    ,
                pub font_variant_alternates:
                    longhands::font_variant_alternates::SpecifiedValue
                    ,
                pub font_variant_east_asian:
                    longhands::font_variant_east_asian::SpecifiedValue
                    ,
                pub font_variant_ligatures:
                    longhands::font_variant_ligatures::SpecifiedValue
                    ,
                pub font_variant_numeric:
                    longhands::font_variant_numeric::SpecifiedValue
                    ,
                pub font_variant_position:
                    longhands::font_variant_position::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub font_variant_caps:
                    &'a longhands::font_variant_caps::SpecifiedValue,
                pub font_variant_alternates:
                    &'a longhands::font_variant_alternates::SpecifiedValue,
                pub font_variant_east_asian:
                    &'a longhands::font_variant_east_asian::SpecifiedValue,
                pub font_variant_ligatures:
                    &'a longhands::font_variant_ligatures::SpecifiedValue,
                pub font_variant_numeric:
                    &'a longhands::font_variant_numeric::SpecifiedValue,
                pub font_variant_position:
                    &'a longhands::font_variant_position::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter(iter: impl Iterator<Item = &'a PropertyDeclaration>) -> Result<Self, ()> {
                // Define all of the expected variables that correspond to the shorthand
                    let mut font_variant_caps =
                        None::< &'a longhands::font_variant_caps::SpecifiedValue>;
                    let mut font_variant_alternates =
                        None::< &'a longhands::font_variant_alternates::SpecifiedValue>;
                    let mut font_variant_east_asian =
                        None::< &'a longhands::font_variant_east_asian::SpecifiedValue>;
                    let mut font_variant_ligatures =
                        None::< &'a longhands::font_variant_ligatures::SpecifiedValue>;
                    let mut font_variant_numeric =
                        None::< &'a longhands::font_variant_numeric::SpecifiedValue>;
                    let mut font_variant_position =
                        None::< &'a longhands::font_variant_position::SpecifiedValue>;

                // Attempt to assign the incoming declarations to the expected variables
                for declaration in iter {
                    match *declaration {
                            PropertyDeclaration::FontVariantCaps(ref value) => {
                                font_variant_caps = Some(value)
                            },
                            PropertyDeclaration::FontVariantAlternates(ref value) => {
                                font_variant_alternates = Some(value)
                            },
                            PropertyDeclaration::FontVariantEastAsian(ref value) => {
                                font_variant_east_asian = Some(value)
                            },
                            PropertyDeclaration::FontVariantLigatures(ref value) => {
                                font_variant_ligatures = Some(value)
                            },
                            PropertyDeclaration::FontVariantNumeric(ref value) => {
                                font_variant_numeric = Some(value)
                            },
                            PropertyDeclaration::FontVariantPosition(ref value) => {
                                font_variant_position = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        font_variant_caps,
                        font_variant_alternates,
                        font_variant_east_asian,
                        font_variant_ligatures,
                        font_variant_numeric,
                        font_variant_position,
                ) {

                    (
                        Some(font_variant_caps),
                        Some(font_variant_alternates),
                        Some(font_variant_east_asian),
                        Some(font_variant_ligatures),
                        Some(font_variant_numeric),
                        Some(font_variant_position),
                    ) =>
                    Ok(LonghandsToSerialize {
                            font_variant_caps,
                            font_variant_alternates,
                            font_variant_east_asian,
                            font_variant_ligatures,
                            font_variant_numeric,
                            font_variant_position,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::FontVariantCaps(
                        longhands.font_variant_caps
                    ));
                    declarations.push(PropertyDeclaration::FontVariantAlternates(
                        longhands.font_variant_alternates
                    ));
                    declarations.push(PropertyDeclaration::FontVariantEastAsian(
                        longhands.font_variant_east_asian
                    ));
                    declarations.push(PropertyDeclaration::FontVariantLigatures(
                        longhands.font_variant_ligatures
                    ));
                    declarations.push(PropertyDeclaration::FontVariantNumeric(
                        longhands.font_variant_numeric
                    ));
                    declarations.push(PropertyDeclaration::FontVariantPosition(
                        longhands.font_variant_position
                    ));
            })
        }

        /// Try to serialize a given shorthand to a string.
        pub fn to_css(declarations: &[&PropertyDeclaration], dest: &mut crate::str::CssStringWriter) -> fmt::Result {
            match LonghandsToSerialize::from_iter(declarations.iter().cloned()) {
                Ok(longhands) => longhands.to_css(&mut CssWriter::new(dest)),
                Err(_) => Ok(())
            }
        }

        
    
    

    use crate::properties::longhands::font_variant_caps;
    use crate::properties::longhands::font_variant_alternates;
    use crate::properties::longhands::font_variant_east_asian;
    use crate::properties::longhands::font_variant_ligatures;
    use crate::properties::longhands::font_variant_numeric;
    use crate::properties::longhands::font_variant_position;
    #[allow(unused_imports)]
    use crate::values::specified::FontVariantLigatures;

    pub fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let mut caps = None;
        let mut alternates = None;
        let mut east_asian = None;
        let mut ligatures = None;
        let mut numeric = None;
        let mut position = None;

        if input.try_parse(|input| input.expect_ident_matching("normal")).is_ok() {
            // Leave the values to None, 'normal' is the initial value for all the sub properties.
        } else if input.try_parse(|input| input.expect_ident_matching("none")).is_ok() {
            // The 'none' value sets 'font-variant-ligatures' to 'none' and resets all other sub properties
            // to their initial value.
            ligatures = Some(FontVariantLigatures::none());
        } else {
            let mut has_custom_value: bool = false;
            loop {
                if input.try_parse(|input| input.expect_ident_matching("normal")).is_ok() ||
                   input.try_parse(|input| input.expect_ident_matching("none")).is_ok() {
                    return Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError))
                }
                if caps.is_none() {
                    if let Ok(value) = input.try_parse(|i| font_variant_caps::parse(context, i)) {
                        has_custom_value = true;
                        caps = Some(value);
                        continue
                    }
                }
                if alternates.is_none() {
                    if let Ok(value) = input.try_parse(|i| font_variant_alternates::parse(context, i)) {
                        has_custom_value = true;
                        alternates = Some(value);
                        continue
                    }
                }
                if east_asian.is_none() {
                    if let Ok(value) = input.try_parse(|i| font_variant_east_asian::parse(context, i)) {
                        has_custom_value = true;
                        east_asian = Some(value);
                        continue
                    }
                }
                if ligatures.is_none() {
                    if let Ok(value) = input.try_parse(|i| font_variant_ligatures::parse(context, i)) {
                        has_custom_value = true;
                        ligatures = Some(value);
                        continue
                    }
                }
                if numeric.is_none() {
                    if let Ok(value) = input.try_parse(|i| font_variant_numeric::parse(context, i)) {
                        has_custom_value = true;
                        numeric = Some(value);
                        continue
                    }
                }
                if position.is_none() {
                    if let Ok(value) = input.try_parse(|i| font_variant_position::parse(context, i)) {
                        has_custom_value = true;
                        position = Some(value);
                        continue
                    }
                }

                break
            }

            if !has_custom_value {
                return Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError))
            }
        }

        Ok(expanded! {
            font_variant_caps: unwrap_or_initial!(font_variant_caps, caps),
            font_variant_alternates: unwrap_or_initial!(font_variant_alternates, alternates),
            font_variant_east_asian: unwrap_or_initial!(font_variant_east_asian, east_asian),
            font_variant_ligatures: unwrap_or_initial!(font_variant_ligatures, ligatures),
            font_variant_numeric: unwrap_or_initial!(font_variant_numeric, numeric),
            font_variant_position: unwrap_or_initial!(font_variant_position, position),
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        #[allow(unused_assignments)]
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {

            let has_none_ligatures =
                self.font_variant_ligatures == &FontVariantLigatures::none();

            const TOTAL_SUBPROPS: usize = 6;
            let mut nb_normals = 0;
            if self.font_variant_caps == &font_variant_caps::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_alternates == &font_variant_alternates::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_east_asian == &font_variant_east_asian::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_ligatures == &font_variant_ligatures::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_numeric == &font_variant_numeric::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_position == &font_variant_position::get_initial_specified_value() {
                nb_normals += 1;
            }


            if nb_normals > 0 && nb_normals == TOTAL_SUBPROPS {
                dest.write_str("normal")?;
            } else if has_none_ligatures {
                if nb_normals == TOTAL_SUBPROPS - 1 {
                    // Serialize to 'none' if 'font-variant-ligatures' is set to 'none' and all other
                    // font feature properties are reset to their initial value.
                    dest.write_str("none")?;
                } else {
                    return Ok(())
                }
            } else {
                let mut has_any = false;
                if self.font_variant_caps != &font_variant_caps::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_caps.to_css(dest)?;
                }
                if self.font_variant_alternates != &font_variant_alternates::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_alternates.to_css(dest)?;
                }
                if self.font_variant_east_asian != &font_variant_east_asian::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_east_asian.to_css(dest)?;
                }
                if self.font_variant_ligatures != &font_variant_ligatures::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_ligatures.to_css(dest)?;
                }
                if self.font_variant_numeric != &font_variant_numeric::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_numeric.to_css(dest)?;
                }
                if self.font_variant_position != &font_variant_position::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_position.to_css(dest)?;
                }
            }

            Ok(())
        }
    }

    }

