/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */




    
    

    /// https://drafts.csswg.org/css-box-3/#propdef-padding
    pub mod padding {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub padding_top:
                    longhands::padding_top::SpecifiedValue
                    ,
                pub padding_right:
                    longhands::padding_right::SpecifiedValue
                    ,
                pub padding_bottom:
                    longhands::padding_bottom::SpecifiedValue
                    ,
                pub padding_left:
                    longhands::padding_left::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub padding_top:
                    &'a longhands::padding_top::SpecifiedValue,
                pub padding_right:
                    &'a longhands::padding_right::SpecifiedValue,
                pub padding_bottom:
                    &'a longhands::padding_bottom::SpecifiedValue,
                pub padding_left:
                    &'a longhands::padding_left::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter(iter: impl Iterator<Item = &'a PropertyDeclaration>) -> Result<Self, ()> {
                // Define all of the expected variables that correspond to the shorthand
                    let mut padding_top =
                        None::< &'a longhands::padding_top::SpecifiedValue>;
                    let mut padding_right =
                        None::< &'a longhands::padding_right::SpecifiedValue>;
                    let mut padding_bottom =
                        None::< &'a longhands::padding_bottom::SpecifiedValue>;
                    let mut padding_left =
                        None::< &'a longhands::padding_left::SpecifiedValue>;

                // Attempt to assign the incoming declarations to the expected variables
                for declaration in iter {
                    match *declaration {
                            PropertyDeclaration::PaddingTop(ref value) => {
                                padding_top = Some(value)
                            },
                            PropertyDeclaration::PaddingRight(ref value) => {
                                padding_right = Some(value)
                            },
                            PropertyDeclaration::PaddingBottom(ref value) => {
                                padding_bottom = Some(value)
                            },
                            PropertyDeclaration::PaddingLeft(ref value) => {
                                padding_left = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        padding_top,
                        padding_right,
                        padding_bottom,
                        padding_left,
                ) {

                    (
                        Some(padding_top),
                        Some(padding_right),
                        Some(padding_bottom),
                        Some(padding_left),
                    ) =>
                    Ok(LonghandsToSerialize {
                            padding_top,
                            padding_right,
                            padding_bottom,
                            padding_left,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::PaddingTop(
                        longhands.padding_top
                    ));
                    declarations.push(PropertyDeclaration::PaddingRight(
                        longhands.padding_right
                    ));
                    declarations.push(PropertyDeclaration::PaddingBottom(
                        longhands.padding_bottom
                    ));
                    declarations.push(PropertyDeclaration::PaddingLeft(
                        longhands.padding_left
                    ));
            })
        }

        /// Try to serialize a given shorthand to a string.
        pub fn to_css(declarations: &[&PropertyDeclaration], dest: &mut crate::str::CssStringWriter) -> fmt::Result {
            match LonghandsToSerialize::from_iter(declarations.iter().cloned()) {
                Ok(longhands) => longhands.to_css(&mut CssWriter::new(dest)),
                Err(_) => Ok(())
            }
        }

        
        #[allow(unused_imports)]
        use crate::parser::Parse;
        use crate::values::generics::rect::Rect;
        #[allow(unused_imports)]
        use crate::values::specified;

        fn parse_value<'i, 't>(
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<Longhands, ParseError<'i>> {
            let rect = Rect::parse_with(context, input, |c, i| -> Result<
                crate::properties::longhands::padding_top::SpecifiedValue,
                ParseError<'i>
            > {
                specified::NonNegativeLengthPercentage::parse_quirky(c, i, specified::AllowQuirks::Yes)
            })?;
            Ok(expanded! {
                    padding_top: rect.0,
                    padding_right: rect.1,
                    padding_bottom: rect.2,
                    padding_left: rect.3,
            })
        }

        impl<'a> ToCss for LonghandsToSerialize<'a> {
            fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result
            where
                W: Write,
            {
                let rect = Rect::new(
                    &self.padding_top,
                    &self.padding_right,
                    &self.padding_bottom,
                    &self.padding_left,
                );
                rect.to_css(dest)
            }
        }
    
    }






    /// https://drafts.csswg.org/css-logical/#propdef-padding-block
    pub mod padding_block {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub padding_block_start:
                    longhands::padding_block_start::SpecifiedValue
                    ,
                pub padding_block_end:
                    longhands::padding_block_end::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub padding_block_start:
                    &'a longhands::padding_block_start::SpecifiedValue,
                pub padding_block_end:
                    &'a longhands::padding_block_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter(iter: impl Iterator<Item = &'a PropertyDeclaration>) -> Result<Self, ()> {
                // Define all of the expected variables that correspond to the shorthand
                    let mut padding_block_start =
                        None::< &'a longhands::padding_block_start::SpecifiedValue>;
                    let mut padding_block_end =
                        None::< &'a longhands::padding_block_end::SpecifiedValue>;

                // Attempt to assign the incoming declarations to the expected variables
                for declaration in iter {
                    match *declaration {
                            PropertyDeclaration::PaddingBlockStart(ref value) => {
                                padding_block_start = Some(value)
                            },
                            PropertyDeclaration::PaddingBlockEnd(ref value) => {
                                padding_block_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        padding_block_start,
                        padding_block_end,
                ) {

                    (
                        Some(padding_block_start),
                        Some(padding_block_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            padding_block_start,
                            padding_block_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::PaddingBlockStart(
                        longhands.padding_block_start
                    ));
                    declarations.push(PropertyDeclaration::PaddingBlockEnd(
                        longhands.padding_block_end
                    ));
            })
        }

        /// Try to serialize a given shorthand to a string.
        pub fn to_css(declarations: &[&PropertyDeclaration], dest: &mut crate::str::CssStringWriter) -> fmt::Result {
            match LonghandsToSerialize::from_iter(declarations.iter().cloned()) {
                Ok(longhands) => longhands.to_css(&mut CssWriter::new(dest)),
                Err(_) => Ok(())
            }
        }

        
    #[allow(unused_imports)]
    use crate::parser::Parse;
    #[allow(unused_imports)]
    use crate::values::specified;

    fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let parse_one = |c: &ParserContext, input: &mut Parser<'i, 't>| -> Result<
            crate::properties::longhands::padding_block_start::SpecifiedValue,
            ParseError<'i>
        > {
            specified::NonNegativeLengthPercentage::parse(c, input)
        };

        let first = parse_one(context, input)?;
        let second =
            input.try_parse(|input| parse_one(context, input)).unwrap_or_else(|_| first.clone());
        Ok(expanded! {
            padding_block_start: first,
            padding_block_end: second,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
            let first = &self.padding_block_start;
            let second = &self.padding_block_end;

            first.to_css(dest)?;
            if first != second {
                dest.write_str(" ")?;
                second.to_css(dest)?;
            }
            Ok(())
        }
    }

    }






    /// https://drafts.csswg.org/css-logical/#propdef-padding-inline
    pub mod padding_inline {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub padding_inline_start:
                    longhands::padding_inline_start::SpecifiedValue
                    ,
                pub padding_inline_end:
                    longhands::padding_inline_end::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub padding_inline_start:
                    &'a longhands::padding_inline_start::SpecifiedValue,
                pub padding_inline_end:
                    &'a longhands::padding_inline_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter(iter: impl Iterator<Item = &'a PropertyDeclaration>) -> Result<Self, ()> {
                // Define all of the expected variables that correspond to the shorthand
                    let mut padding_inline_start =
                        None::< &'a longhands::padding_inline_start::SpecifiedValue>;
                    let mut padding_inline_end =
                        None::< &'a longhands::padding_inline_end::SpecifiedValue>;

                // Attempt to assign the incoming declarations to the expected variables
                for declaration in iter {
                    match *declaration {
                            PropertyDeclaration::PaddingInlineStart(ref value) => {
                                padding_inline_start = Some(value)
                            },
                            PropertyDeclaration::PaddingInlineEnd(ref value) => {
                                padding_inline_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        padding_inline_start,
                        padding_inline_end,
                ) {

                    (
                        Some(padding_inline_start),
                        Some(padding_inline_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            padding_inline_start,
                            padding_inline_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::PaddingInlineStart(
                        longhands.padding_inline_start
                    ));
                    declarations.push(PropertyDeclaration::PaddingInlineEnd(
                        longhands.padding_inline_end
                    ));
            })
        }

        /// Try to serialize a given shorthand to a string.
        pub fn to_css(declarations: &[&PropertyDeclaration], dest: &mut crate::str::CssStringWriter) -> fmt::Result {
            match LonghandsToSerialize::from_iter(declarations.iter().cloned()) {
                Ok(longhands) => longhands.to_css(&mut CssWriter::new(dest)),
                Err(_) => Ok(())
            }
        }

        
    #[allow(unused_imports)]
    use crate::parser::Parse;
    #[allow(unused_imports)]
    use crate::values::specified;

    fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let parse_one = |c: &ParserContext, input: &mut Parser<'i, 't>| -> Result<
            crate::properties::longhands::padding_inline_start::SpecifiedValue,
            ParseError<'i>
        > {
            specified::NonNegativeLengthPercentage::parse(c, input)
        };

        let first = parse_one(context, input)?;
        let second =
            input.try_parse(|input| parse_one(context, input)).unwrap_or_else(|_| first.clone());
        Ok(expanded! {
            padding_inline_start: first,
            padding_inline_end: second,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
            let first = &self.padding_inline_start;
            let second = &self.padding_inline_end;

            first.to_css(dest)?;
            if first != second {
                dest.write_str(" ")?;
                second.to_css(dest)?;
            }
            Ok(())
        }
    }

    }




    
    

    /// https://drafts.csswg.org/css-scroll-snap-1/#propdef-scroll-padding
    pub mod scroll_padding {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub scroll_padding_top:
                    longhands::scroll_padding_top::SpecifiedValue
                    ,
                pub scroll_padding_right:
                    longhands::scroll_padding_right::SpecifiedValue
                    ,
                pub scroll_padding_bottom:
                    longhands::scroll_padding_bottom::SpecifiedValue
                    ,
                pub scroll_padding_left:
                    longhands::scroll_padding_left::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub scroll_padding_top:
                    &'a longhands::scroll_padding_top::SpecifiedValue,
                pub scroll_padding_right:
                    &'a longhands::scroll_padding_right::SpecifiedValue,
                pub scroll_padding_bottom:
                    &'a longhands::scroll_padding_bottom::SpecifiedValue,
                pub scroll_padding_left:
                    &'a longhands::scroll_padding_left::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter(iter: impl Iterator<Item = &'a PropertyDeclaration>) -> Result<Self, ()> {
                // Define all of the expected variables that correspond to the shorthand
                    let mut scroll_padding_top =
                        None::< &'a longhands::scroll_padding_top::SpecifiedValue>;
                    let mut scroll_padding_right =
                        None::< &'a longhands::scroll_padding_right::SpecifiedValue>;
                    let mut scroll_padding_bottom =
                        None::< &'a longhands::scroll_padding_bottom::SpecifiedValue>;
                    let mut scroll_padding_left =
                        None::< &'a longhands::scroll_padding_left::SpecifiedValue>;

                // Attempt to assign the incoming declarations to the expected variables
                for declaration in iter {
                    match *declaration {
                            PropertyDeclaration::ScrollPaddingTop(ref value) => {
                                scroll_padding_top = Some(value)
                            },
                            PropertyDeclaration::ScrollPaddingRight(ref value) => {
                                scroll_padding_right = Some(value)
                            },
                            PropertyDeclaration::ScrollPaddingBottom(ref value) => {
                                scroll_padding_bottom = Some(value)
                            },
                            PropertyDeclaration::ScrollPaddingLeft(ref value) => {
                                scroll_padding_left = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        scroll_padding_top,
                        scroll_padding_right,
                        scroll_padding_bottom,
                        scroll_padding_left,
                ) {

                    (
                        Some(scroll_padding_top),
                        Some(scroll_padding_right),
                        Some(scroll_padding_bottom),
                        Some(scroll_padding_left),
                    ) =>
                    Ok(LonghandsToSerialize {
                            scroll_padding_top,
                            scroll_padding_right,
                            scroll_padding_bottom,
                            scroll_padding_left,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ScrollPaddingTop(
                        longhands.scroll_padding_top
                    ));
                    declarations.push(PropertyDeclaration::ScrollPaddingRight(
                        longhands.scroll_padding_right
                    ));
                    declarations.push(PropertyDeclaration::ScrollPaddingBottom(
                        longhands.scroll_padding_bottom
                    ));
                    declarations.push(PropertyDeclaration::ScrollPaddingLeft(
                        longhands.scroll_padding_left
                    ));
            })
        }

        /// Try to serialize a given shorthand to a string.
        pub fn to_css(declarations: &[&PropertyDeclaration], dest: &mut crate::str::CssStringWriter) -> fmt::Result {
            match LonghandsToSerialize::from_iter(declarations.iter().cloned()) {
                Ok(longhands) => longhands.to_css(&mut CssWriter::new(dest)),
                Err(_) => Ok(())
            }
        }

        
        #[allow(unused_imports)]
        use crate::parser::Parse;
        use crate::values::generics::rect::Rect;
        #[allow(unused_imports)]
        use crate::values::specified;

        fn parse_value<'i, 't>(
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<Longhands, ParseError<'i>> {
            let rect = Rect::parse_with(context, input, |c, i| -> Result<
                crate::properties::longhands::scroll_padding_top::SpecifiedValue,
                ParseError<'i>
            > {
                specified::NonNegativeLengthPercentageOrAuto::parse(c, i)
            })?;
            Ok(expanded! {
                    scroll_padding_top: rect.0,
                    scroll_padding_right: rect.1,
                    scroll_padding_bottom: rect.2,
                    scroll_padding_left: rect.3,
            })
        }

        impl<'a> ToCss for LonghandsToSerialize<'a> {
            fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result
            where
                W: Write,
            {
                let rect = Rect::new(
                    &self.scroll_padding_top,
                    &self.scroll_padding_right,
                    &self.scroll_padding_bottom,
                    &self.scroll_padding_left,
                );
                rect.to_css(dest)
            }
        }
    
    }






    /// https://drafts.csswg.org/css-scroll-snap-1/#propdef-scroll-padding-block
    pub mod scroll_padding_block {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub scroll_padding_block_start:
                    longhands::scroll_padding_block_start::SpecifiedValue
                    ,
                pub scroll_padding_block_end:
                    longhands::scroll_padding_block_end::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub scroll_padding_block_start:
                    &'a longhands::scroll_padding_block_start::SpecifiedValue,
                pub scroll_padding_block_end:
                    &'a longhands::scroll_padding_block_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter(iter: impl Iterator<Item = &'a PropertyDeclaration>) -> Result<Self, ()> {
                // Define all of the expected variables that correspond to the shorthand
                    let mut scroll_padding_block_start =
                        None::< &'a longhands::scroll_padding_block_start::SpecifiedValue>;
                    let mut scroll_padding_block_end =
                        None::< &'a longhands::scroll_padding_block_end::SpecifiedValue>;

                // Attempt to assign the incoming declarations to the expected variables
                for declaration in iter {
                    match *declaration {
                            PropertyDeclaration::ScrollPaddingBlockStart(ref value) => {
                                scroll_padding_block_start = Some(value)
                            },
                            PropertyDeclaration::ScrollPaddingBlockEnd(ref value) => {
                                scroll_padding_block_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        scroll_padding_block_start,
                        scroll_padding_block_end,
                ) {

                    (
                        Some(scroll_padding_block_start),
                        Some(scroll_padding_block_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            scroll_padding_block_start,
                            scroll_padding_block_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ScrollPaddingBlockStart(
                        longhands.scroll_padding_block_start
                    ));
                    declarations.push(PropertyDeclaration::ScrollPaddingBlockEnd(
                        longhands.scroll_padding_block_end
                    ));
            })
        }

        /// Try to serialize a given shorthand to a string.
        pub fn to_css(declarations: &[&PropertyDeclaration], dest: &mut crate::str::CssStringWriter) -> fmt::Result {
            match LonghandsToSerialize::from_iter(declarations.iter().cloned()) {
                Ok(longhands) => longhands.to_css(&mut CssWriter::new(dest)),
                Err(_) => Ok(())
            }
        }

        
    #[allow(unused_imports)]
    use crate::parser::Parse;
    #[allow(unused_imports)]
    use crate::values::specified;

    fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let parse_one = |c: &ParserContext, input: &mut Parser<'i, 't>| -> Result<
            crate::properties::longhands::scroll_padding_block_start::SpecifiedValue,
            ParseError<'i>
        > {
            specified::NonNegativeLengthPercentageOrAuto::parse(c, input)
        };

        let first = parse_one(context, input)?;
        let second =
            input.try_parse(|input| parse_one(context, input)).unwrap_or_else(|_| first.clone());
        Ok(expanded! {
            scroll_padding_block_start: first,
            scroll_padding_block_end: second,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
            let first = &self.scroll_padding_block_start;
            let second = &self.scroll_padding_block_end;

            first.to_css(dest)?;
            if first != second {
                dest.write_str(" ")?;
                second.to_css(dest)?;
            }
            Ok(())
        }
    }

    }






    /// https://drafts.csswg.org/css-scroll-snap-1/#propdef-scroll-padding-inline
    pub mod scroll_padding_inline {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub scroll_padding_inline_start:
                    longhands::scroll_padding_inline_start::SpecifiedValue
                    ,
                pub scroll_padding_inline_end:
                    longhands::scroll_padding_inline_end::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub scroll_padding_inline_start:
                    &'a longhands::scroll_padding_inline_start::SpecifiedValue,
                pub scroll_padding_inline_end:
                    &'a longhands::scroll_padding_inline_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter(iter: impl Iterator<Item = &'a PropertyDeclaration>) -> Result<Self, ()> {
                // Define all of the expected variables that correspond to the shorthand
                    let mut scroll_padding_inline_start =
                        None::< &'a longhands::scroll_padding_inline_start::SpecifiedValue>;
                    let mut scroll_padding_inline_end =
                        None::< &'a longhands::scroll_padding_inline_end::SpecifiedValue>;

                // Attempt to assign the incoming declarations to the expected variables
                for declaration in iter {
                    match *declaration {
                            PropertyDeclaration::ScrollPaddingInlineStart(ref value) => {
                                scroll_padding_inline_start = Some(value)
                            },
                            PropertyDeclaration::ScrollPaddingInlineEnd(ref value) => {
                                scroll_padding_inline_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        scroll_padding_inline_start,
                        scroll_padding_inline_end,
                ) {

                    (
                        Some(scroll_padding_inline_start),
                        Some(scroll_padding_inline_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            scroll_padding_inline_start,
                            scroll_padding_inline_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ScrollPaddingInlineStart(
                        longhands.scroll_padding_inline_start
                    ));
                    declarations.push(PropertyDeclaration::ScrollPaddingInlineEnd(
                        longhands.scroll_padding_inline_end
                    ));
            })
        }

        /// Try to serialize a given shorthand to a string.
        pub fn to_css(declarations: &[&PropertyDeclaration], dest: &mut crate::str::CssStringWriter) -> fmt::Result {
            match LonghandsToSerialize::from_iter(declarations.iter().cloned()) {
                Ok(longhands) => longhands.to_css(&mut CssWriter::new(dest)),
                Err(_) => Ok(())
            }
        }

        
    #[allow(unused_imports)]
    use crate::parser::Parse;
    #[allow(unused_imports)]
    use crate::values::specified;

    fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let parse_one = |c: &ParserContext, input: &mut Parser<'i, 't>| -> Result<
            crate::properties::longhands::scroll_padding_inline_start::SpecifiedValue,
            ParseError<'i>
        > {
            specified::NonNegativeLengthPercentageOrAuto::parse(c, input)
        };

        let first = parse_one(context, input)?;
        let second =
            input.try_parse(|input| parse_one(context, input)).unwrap_or_else(|_| first.clone());
        Ok(expanded! {
            scroll_padding_inline_start: first,
            scroll_padding_inline_end: second,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
            let first = &self.scroll_padding_inline_start;
            let second = &self.scroll_padding_inline_end;

            first.to_css(dest)?;
            if first != second {
                dest.write_str(" ")?;
                second.to_css(dest)?;
            }
            Ok(())
        }
    }

    }


