// -*- mode: Rust -*-

// AUTOGENERATED BY glean_parser.  DO NOT EDIT.

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

pub mod browser_engagement {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from browser.engagement.active_ticks
    ///
    /// The number of five-second intervals ('ticks') the user was considered
    /// 'active'.
    /// 
    /// 'active' means keyboard or mouse interaction with the application.
    /// It doesn't take into account whether or not the window has focus or is in
    /// the foreground, only if it is receiving these interaction events.
    /// 
    /// Migrated from Telemetry's `browser.engagement.active_ticks`.
    pub static active_ticks: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(1.into(), CommonMetricData {
            name: "active_ticks".into(),
            category: "browser.engagement".into(),
            send_in_pings: vec!["baseline".into(), "metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from browser.engagement.uri_count
    ///
    /// The number of total non-unique http(s) URIs visited, including page
    /// reloads, after the session has been restored. URIs on minimized or
    /// background tabs may also be counted. Private browsing uris are included.
    /// 
    /// Migrated from Telemetry's
    /// `browser.engagement.total_uri_count_normal_and_private_mode`.
    pub static uri_count: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(2.into(), CommonMetricData {
            name: "uri_count".into(),
            category: "browser.engagement".into(),
            send_in_pings: vec!["baseline".into(), "metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod gmp {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from gmp.update_xml_fetch_result
    ///
    /// The result of Gecko fetching an update.xml from Balrog. This captures 3
    /// different data points: success or failure of the request, if cert pinning or
    /// content signatures were used to verify the result, and the reason for failure,
    /// if the request failed.
    pub static update_xml_fetch_result: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(3.into(), CommonMetricData {
            name: "update_xml_fetch_result".into(),
            category: "gmp".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["cert_pin_abort".into(), "cert_pin_failed".into(), "cert_pin_invalid".into(), "cert_pin_missing_data".into(), "cert_pin_net_request_error".into(), "cert_pin_net_timeout".into(), "cert_pin_success".into(), "cert_pin_unknown_error".into(), "cert_pin_xml_parse_error".into(), "content_sig_abort".into(), "content_sig_failed".into(), "content_sig_invalid".into(), "content_sig_missing_data".into(), "content_sig_net_request_error".into(), "content_sig_net_timeout".into(), "content_sig_success".into(), "content_sig_unknown_error".into(), "content_sig_xml_parse_error".into()]))
    });

}
pub mod perf {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[derive(Default, Debug, Clone, Hash, Eq, PartialEq)]
    pub struct PageLoadExtra {
        pub fcp_time: Option<u32>,
        pub js_exec_time: Option<u32>,
        pub load_time: Option<u32>,
        pub load_type: Option<String>,
        pub response_time: Option<u32>,
    }

    impl ExtraKeys for PageLoadExtra {
        const ALLOWED_KEYS: &'static [&'static str] = &["fcp_time", "js_exec_time", "load_time", "load_type", "response_time"];

        fn into_ffi_extra(self) -> ::std::collections::HashMap<i32, String> {
          let mut map = ::std::collections::HashMap::new();
          self.fcp_time.and_then(|val| map.insert(0, val.to_string()));
          self.js_exec_time.and_then(|val| map.insert(1, val.to_string()));
          self.load_time.and_then(|val| map.insert(2, val.to_string()));
          self.load_type.and_then(|val| map.insert(3, val.to_string()));
          self.response_time.and_then(|val| map.insert(4, val.to_string()));
          map
        }
    }
    #[allow(non_upper_case_globals)]
    /// generated from perf.page_load
    ///
    /// Recorded when a top level content document has been loaded.
    pub static page_load: Lazy<EventMetric<PageLoadExtra>> = Lazy::new(|| {
        EventMetric::new(4.into(), CommonMetricData {
            name: "page_load".into(),
            category: "perf".into(),
            send_in_pings: vec!["events".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod paint {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from paint.build_displaylist_time
    ///
    /// The time to build a Gecko display list.
    pub static build_displaylist_time: Lazy<TimingDistributionMetric> = Lazy::new(|| {
        TimingDistributionMetric::new(5.into(), CommonMetricData {
            name: "build_displaylist_time".into(),
            category: "paint".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Nanosecond)
    });

}
pub mod wr {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from wr.rasterize_glyphs_time
    ///
    /// The time to rasterize glyphs for consumption by WebRender.
    pub static rasterize_glyphs_time: Lazy<TimingDistributionMetric> = Lazy::new(|| {
        TimingDistributionMetric::new(6.into(), CommonMetricData {
            name: "rasterize_glyphs_time".into(),
            category: "wr".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Microsecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from wr.framebuild_time
    ///
    /// The time to rasterize glyphs for consumption by WebRender.
    pub static framebuild_time: Lazy<TimingDistributionMetric> = Lazy::new(|| {
        TimingDistributionMetric::new(7.into(), CommonMetricData {
            name: "framebuild_time".into(),
            category: "wr".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Microsecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from wr.scenebuild_time
    ///
    /// WebRender scene build time.
    pub static scenebuild_time: Lazy<TimingDistributionMetric> = Lazy::new(|| {
        TimingDistributionMetric::new(8.into(), CommonMetricData {
            name: "scenebuild_time".into(),
            category: "wr".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Microsecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from wr.sceneswap_time
    ///
    /// WebRender scene swap time.
    pub static sceneswap_time: Lazy<TimingDistributionMetric> = Lazy::new(|| {
        TimingDistributionMetric::new(9.into(), CommonMetricData {
            name: "sceneswap_time".into(),
            category: "wr".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Microsecond)
    });

}
pub mod extensions {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from extensions.startup_cache_load_time
    ///
    /// Time to load and deserialize the extensions startupCache data.
    pub static startup_cache_load_time: Lazy<TimespanMetric> = Lazy::new(|| {
        TimespanMetric::new(10.into(), CommonMetricData {
            name: "startup_cache_load_time".into(),
            category: "extensions".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Millisecond)
    });

}
pub mod fog {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from fog.initialization
    ///
    /// Time the FOG initialization takes.
    pub static initialization: Lazy<TimespanMetric> = Lazy::new(|| {
        TimespanMetric::new(11.into(), CommonMetricData {
            name: "initialization".into(),
            category: "fog".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Nanosecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from fog.failed_idle_registration
    ///
    /// True if we failed to register with the idle service. Absent otherwise.
    /// Means IPC probably isn't working well.
    /// Child-process data will likely be absent, or incomplete.
    pub static failed_idle_registration: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(12.into(), CommonMetricData {
            name: "failed_idle_registration".into(),
            category: "fog".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod fog_ipc {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from fog.ipc.replay_failures
    ///
    /// The number of times the ipc buffer failed to be replayed in the
    /// parent process.
    pub static replay_failures: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(13.into(), CommonMetricData {
            name: "replay_failures".into(),
            category: "fog.ipc".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from fog.ipc.buffer_sizes
    ///
    /// The number and size of the IPC buffers being received over FOG IPC.
    pub static buffer_sizes: Lazy<MemoryDistributionMetric> = Lazy::new(|| {
        MemoryDistributionMetric::new(14.into(), CommonMetricData {
            name: "buffer_sizes".into(),
            category: "fog.ipc".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, MemoryUnit::Byte)
    });

    #[allow(non_upper_case_globals)]
    /// generated from fog.ipc.flush_durations
    ///
    /// The length of time between asking the child processes for their
    /// IPC buffers and all of them being received by the parent.
    pub static flush_durations: Lazy<TimingDistributionMetric> = Lazy::new(|| {
        TimingDistributionMetric::new(15.into(), CommonMetricData {
            name: "flush_durations".into(),
            category: "fog.ipc".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Microsecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from fog.ipc.flush_failures
    ///
    /// The number of times we failed to flush all non-parent-process data,
    /// throwing even partial results into the trash.
    /// If this number is high, we might consider writing custom `MozPromise`-
    /// handling code instead of using `MozPromise::All`.
    pub static flush_failures: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(16.into(), CommonMetricData {
            name: "flush_failures".into(),
            category: "fog.ipc".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from fog.ipc.shutdown_registration_failures
    ///
    /// The number of times we tried to register shutdown flush routines for
    /// content child processes, and failed (probably because there was no main
    /// thread).
    /// As a result there may be data loss from content child processes.
    /// Large or rising number of clients experiencing this indicates we should
    /// perhaps refactor content child shutdown in FOG to try harder to register
    /// flush operations.
    /// Will likely be obsoleted by bug 1641989.
    pub static shutdown_registration_failures: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(17.into(), CommonMetricData {
            name: "shutdown_registration_failures".into(),
            category: "fog.ipc".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod test_only {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from test_only.bad_code
    ///
    /// Number of times we encountered bad code.
    /// This is a test-only metric.
    pub static bad_code: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(18.into(), CommonMetricData {
            name: "bad_code".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.can_we_time_it
    ///
    /// Test metric for a timespan.
    /// This is a test-only metric.
    pub static can_we_time_it: Lazy<TimespanMetric> = Lazy::new(|| {
        TimespanMetric::new(19.into(), CommonMetricData {
            name: "can_we_time_it".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Nanosecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.cheesy_string
    ///
    /// Only the cheesiest of strings.
    /// This is a test-only metric.
    pub static cheesy_string: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(20.into(), CommonMetricData {
            name: "cheesy_string".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.cheesy_string_list
    ///
    /// Only the cheesiest of strings. In list form!
    /// This is a test-only metric.
    pub static cheesy_string_list: Lazy<StringListMetric> = Lazy::new(|| {
        StringListMetric::new(21.into(), CommonMetricData {
            name: "cheesy_string_list".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.what_a_date
    ///
    /// ...To be writing FOG code. This is a test-only metric.
    pub static what_a_date: Lazy<DatetimeMetric> = Lazy::new(|| {
        DatetimeMetric::new(22.into(), CommonMetricData {
            name: "what_a_date".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Second)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.what_id_it
    ///
    /// Just a UUID.
    /// This is a test-only metric.
    pub static what_id_it: Lazy<UuidMetric> = Lazy::new(|| {
        UuidMetric::new(23.into(), CommonMetricData {
            name: "what_id_it".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.can_we_flag_it
    ///
    /// Test metric for a boolean.
    /// This is a test-only metric.
    pub static can_we_flag_it: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(24.into(), CommonMetricData {
            name: "can_we_flag_it".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.do_you_remember
    ///
    /// They say it's the second thing to go.
    /// This is a test-only metric.
    pub static do_you_remember: Lazy<MemoryDistributionMetric> = Lazy::new(|| {
        MemoryDistributionMetric::new(25.into(), CommonMetricData {
            name: "do_you_remember".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, MemoryUnit::Megabyte)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.what_time_is_it
    ///
    /// Adheres to at least two of the top ten fallacies programmers believe
    /// about time.
    /// This is a test-only metric.
    pub static what_time_is_it: Lazy<TimingDistributionMetric> = Lazy::new(|| {
        TimingDistributionMetric::new(26.into(), CommonMetricData {
            name: "what_time_is_it".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Microsecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.mabels_kitchen_counters
    ///
    /// Counts Mabels labeled by their kitchen counters.
    /// This is a test-only metric.
    pub static mabels_kitchen_counters: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(27.into(), CommonMetricData {
            name: "mabels_kitchen_counters".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, None)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.mabels_bathroom_counters
    ///
    /// Counts Mabels labeled by their bathroom counters.
    /// This is a test-only metric.
    pub static mabels_bathroom_counters: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(28.into(), CommonMetricData {
            name: "mabels_bathroom_counters".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, None)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.mabels_like_balloons
    ///
    /// Does the labeled Mabel like balloons?
    /// This is a test-only metric.
    pub static mabels_like_balloons: Lazy<LabeledMetric<LabeledBooleanMetric>> = Lazy::new(|| {
        LabeledMetric::new(29.into(), CommonMetricData {
            name: "mabels_like_balloons".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, None)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.mabels_balloon_strings
    ///
    /// What do the labeled Mabel's liked balloons' strings say?
    /// This is a test-only metric.
    pub static mabels_balloon_strings: Lazy<LabeledMetric<LabeledStringMetric>> = Lazy::new(|| {
        LabeledMetric::new(30.into(), CommonMetricData {
            name: "mabels_balloon_strings".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, None)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.mirror_time
    ///
    /// Mirrored metric for a timespan.
    /// This is a test-only metric.
    pub static mirror_time: Lazy<TimespanMetric> = Lazy::new(|| {
        TimespanMetric::new(31.into(), CommonMetricData {
            name: "mirror_time".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Nanosecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.mirror_time_nanos
    ///
    /// Mirrored metric for a timespan.
    /// This is a test-only metric.
    pub static mirror_time_nanos: Lazy<TimespanMetric> = Lazy::new(|| {
        TimespanMetric::new(32.into(), CommonMetricData {
            name: "mirror_time_nanos".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Nanosecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.mirrors_for_labeled_bools
    ///
    /// Mirrored metric.
    /// This is a test-only metric.
    pub static mirrors_for_labeled_bools: Lazy<LabeledMetric<LabeledBooleanMetric>> = Lazy::new(|| {
        LabeledMetric::new(33.into(), CommonMetricData {
            name: "mirrors_for_labeled_bools".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, None)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.one_ping_one_bool
    ///
    /// One bool for one ping only.
    /// This is a test-only metric.
    pub static one_ping_one_bool: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(34.into(), CommonMetricData {
            name: "one_ping_one_bool".into(),
            category: "test_only".into(),
            send_in_pings: vec!["one-ping-only".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.meaning_of_life
    ///
    /// Measures the one true answer to the Ultimate Question of Life,
    /// the Universe, and Everything.
    /// Approximately.
    /// This is a test-only metric.
    pub static meaning_of_life: Lazy<QuantityMetric> = Lazy::new(|| {
        QuantityMetric::new(35.into(), CommonMetricData {
            name: "meaning_of_life".into(),
            category: "test_only".into(),
            send_in_pings: vec!["test-ping".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod test_only_ipc {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_counter
    ///
    /// This is a test-only metric.
    /// Just counting things.
    pub static a_counter: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(36.into(), CommonMetricData {
            name: "a_counter".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_bool
    ///
    /// This is a test-only metric.
    /// Just flagging things.
    pub static a_bool: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(37.into(), CommonMetricData {
            name: "a_bool".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_date
    ///
    /// This is a test-only metric.
    /// Just putting things on the calendar.
    pub static a_date: Lazy<DatetimeMetric> = Lazy::new(|| {
        DatetimeMetric::new(38.into(), CommonMetricData {
            name: "a_date".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Second)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_string
    ///
    /// This is a test-only metric.
    /// Just setting some strings.
    pub static a_string: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(39.into(), CommonMetricData {
            name: "a_string".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_memory_dist
    ///
    /// This is a test-only metric.
    /// Just measuring memory.
    pub static a_memory_dist: Lazy<MemoryDistributionMetric> = Lazy::new(|| {
        MemoryDistributionMetric::new(40.into(), CommonMetricData {
            name: "a_memory_dist".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, MemoryUnit::Kilobyte)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_timing_dist
    ///
    /// This is a test-only metric.
    /// Just measuring time.
    pub static a_timing_dist: Lazy<TimingDistributionMetric> = Lazy::new(|| {
        TimingDistributionMetric::new(41.into(), CommonMetricData {
            name: "a_timing_dist".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, TimeUnit::Nanosecond)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_custom_dist
    ///
    /// This is a test-only metric.
    /// Just measuring samples.
    pub static a_custom_dist: Lazy<CustomDistributionMetric> = Lazy::new(|| {
        CustomDistributionMetric::new(42.into(), CommonMetricData {
            name: "a_custom_dist".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, 1, 2147483646, 10, HistogramType::Linear)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_string_list
    ///
    /// This is a test-only metric.
    /// Just appending some strings.
    pub static a_string_list: Lazy<StringListMetric> = Lazy::new(|| {
        StringListMetric::new(43.into(), CommonMetricData {
            name: "a_string_list".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[derive(Default, Debug, Clone, Hash, Eq, PartialEq)]
    pub struct AnEventExtra {
        pub extra1: Option<String>,
        pub extra2: Option<String>,
    }

    impl ExtraKeys for AnEventExtra {
        const ALLOWED_KEYS: &'static [&'static str] = &["extra1", "extra2"];

        fn into_ffi_extra(self) -> ::std::collections::HashMap<i32, String> {
          let mut map = ::std::collections::HashMap::new();
          self.extra1.and_then(|val| map.insert(0, val.to_string()));
          self.extra2.and_then(|val| map.insert(1, val.to_string()));
          map
        }
    }
    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.an_event
    ///
    /// This is a test-only metric.
    /// Just recording some events.
    pub static an_event: Lazy<EventMetric<AnEventExtra>> = Lazy::new(|| {
        EventMetric::new(44.into(), CommonMetricData {
            name: "an_event".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[derive(Default, Debug, Clone, Hash, Eq, PartialEq)]
    pub struct EventWithExtraExtra {
        pub extra1: Option<String>,
        pub extra2: Option<u32>,
        pub extra3_longer_name: Option<bool>,
    }

    impl ExtraKeys for EventWithExtraExtra {
        const ALLOWED_KEYS: &'static [&'static str] = &["extra1", "extra2", "extra3_longer_name"];

        fn into_ffi_extra(self) -> ::std::collections::HashMap<i32, String> {
          let mut map = ::std::collections::HashMap::new();
          self.extra1.and_then(|val| map.insert(0, val.to_string()));
          self.extra2.and_then(|val| map.insert(1, val.to_string()));
          self.extra3_longer_name.and_then(|val| map.insert(2, val.to_string()));
          map
        }
    }
    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.event_with_extra
    ///
    /// This is a test-only metric.
    /// Just recording some events.
    pub static event_with_extra: Lazy<EventMetric<EventWithExtraExtra>> = Lazy::new(|| {
        EventMetric::new(45.into(), CommonMetricData {
            name: "event_with_extra".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.no_extra_event
    ///
    /// This is a test-only metric.
    /// Just recording some events without the extra fuss.
    pub static no_extra_event: Lazy<EventMetric<NoExtraKeys>> = Lazy::new(|| {
        EventMetric::new(46.into(), CommonMetricData {
            name: "no_extra_event".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_uuid
    ///
    /// This is a test-only metric.
    /// Just recording some unique identifiers.
    pub static a_uuid: Lazy<UuidMetric> = Lazy::new(|| {
        UuidMetric::new(47.into(), CommonMetricData {
            name: "a_uuid".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_labeled_counter
    ///
    /// This is a test-only metric.
    /// Just counting labeled things.
    pub static a_labeled_counter: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(48.into(), CommonMetricData {
            name: "a_labeled_counter".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, None)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.another_labeled_counter
    ///
    /// This is a test-only metric.
    /// Just another metric counting labeled things.
    pub static another_labeled_counter: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(49.into(), CommonMetricData {
            name: "another_labeled_counter".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, None)
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_quantity
    ///
    /// This is a test-only metric.
    /// Just quantifying things.
    pub static a_quantity: Lazy<QuantityMetric> = Lazy::new(|| {
        QuantityMetric::new(50.into(), CommonMetricData {
            name: "a_quantity".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.irate
    ///
    /// This is a test-only metric.
    /// A rate that isn't happy about it.
    pub static irate: Lazy<RateMetric> = Lazy::new(|| {
        RateMetric::new(51.into(), CommonMetricData {
            name: "irate".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.rate_with_external_denominator
    ///
    /// This is a test-only metric.
    /// A rate with a denominator that is Out There.
    pub static rate_with_external_denominator: Lazy<NumeratorMetric> = Lazy::new(|| {
        NumeratorMetric::new(52.into(), CommonMetricData {
            name: "rate_with_external_denominator".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.an_external_denominator
    ///
    /// This is a test-only metric.
    /// A denominator not from around here.
    pub static an_external_denominator: Lazy<DenominatorMetric> = Lazy::new(|| {
        DenominatorMetric::new(53.into(), CommonMetricData {
            name: "an_external_denominator".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, vec![CommonMetricData {name: "rate_with_external_denominator".into(), category: "test_only.ipc".into(), send_in_pings: vec!["store1".into()], lifetime: Lifetime::Ping, disabled: false,  ..Default::default()}])
    });

    #[allow(non_upper_case_globals)]
    /// generated from test_only.ipc.a_url
    ///
    /// This is a test-only metric.
    /// Just setting some Urls.
    pub static a_url: Lazy<UrlMetric> = Lazy::new(|| {
        UrlMetric::new(54.into(), CommonMetricData {
            name: "a_url".into(),
            category: "test_only.ipc".into(),
            send_in_pings: vec!["store1".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod power {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from power.cpu_time_bogus_values
    ///
    /// Impossibly large CPU time values that were discarded.
    pub static cpu_time_bogus_values: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(55.into(), CommonMetricData {
            name: "cpu_time_bogus_values".into(),
            category: "power".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.cpu_time_per_process_type_ms
    ///
    /// CPU time used by each process type in ms.
    pub static cpu_time_per_process_type_ms: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(56.into(), CommonMetricData {
            name: "cpu_time_per_process_type_ms".into(),
            category: "power".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["extension".into(), "gmplugin".into(), "gpu".into(), "parent.active".into(), "parent.active.playing-audio".into(), "parent.active.playing-video".into(), "parent.inactive".into(), "parent.inactive.playing-audio".into(), "parent.inactive.playing-video".into(), "prealloc".into(), "privilegedabout".into(), "rdd".into(), "socket".into(), "utility".into(), "web.background".into(), "web.background-perceivable".into(), "web.foreground".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.gpu_time_per_process_type_ms
    ///
    /// GPU time used by each process type in ms.
    pub static gpu_time_per_process_type_ms: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(57.into(), CommonMetricData {
            name: "gpu_time_per_process_type_ms".into(),
            category: "power".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["extension".into(), "gmplugin".into(), "gpu".into(), "parent.active".into(), "parent.active.playing-audio".into(), "parent.active.playing-video".into(), "parent.inactive".into(), "parent.inactive.playing-audio".into(), "parent.inactive.playing-video".into(), "prealloc".into(), "privilegedabout".into(), "rdd".into(), "socket".into(), "utility".into(), "web.background".into(), "web.background-perceivable".into(), "web.foreground".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.gpu_time_bogus_values
    ///
    /// Impossibly large GPU time values that were discarded.
    pub static gpu_time_bogus_values: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(58.into(), CommonMetricData {
            name: "gpu_time_bogus_values".into(),
            category: "power".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.wakeups_per_process_type
    ///
    /// How many times threads woke up and could have woken up a CPU core. Broken down
    /// by process type.
    pub static wakeups_per_process_type: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(59.into(), CommonMetricData {
            name: "wakeups_per_process_type".into(),
            category: "power".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["extension".into(), "gmplugin".into(), "gpu".into(), "parent.active".into(), "parent.active.playing-audio".into(), "parent.active.playing-video".into(), "parent.inactive".into(), "parent.inactive.playing-audio".into(), "parent.inactive.playing-video".into(), "prealloc".into(), "privilegedabout".into(), "rdd".into(), "socket".into(), "utility".into(), "web.background".into(), "web.background-perceivable".into(), "web.foreground".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.total_cpu_time_ms
    ///
    /// Total CPU time used by all processes in ms.
    pub static total_cpu_time_ms: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(60.into(), CommonMetricData {
            name: "total_cpu_time_ms".into(),
            category: "power".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.total_gpu_time_ms
    ///
    /// Total GPU time used by all processes in ms.
    pub static total_gpu_time_ms: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(61.into(), CommonMetricData {
            name: "total_gpu_time_ms".into(),
            category: "power".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.total_thread_wakeups
    ///
    /// How many times threads woke up and could have woken up a CPU core.
    pub static total_thread_wakeups: Lazy<CounterMetric> = Lazy::new(|| {
        CounterMetric::new(62.into(), CommonMetricData {
            name: "total_thread_wakeups".into(),
            category: "power".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod power_wakeups_per_thread {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from power.wakeups_per_thread.parent_active
    ///
    /// How many times threads woke up and could have woken up a CPU core. Broken down
    /// by thread name for a given process type.
    pub static parent_active: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(63.into(), CommonMetricData {
            name: "parent_active".into(),
            category: "power.wakeups_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.wakeups_per_thread.parent_inactive
    ///
    /// How many times threads woke up and could have woken up a CPU core. Broken down
    /// by thread name for a given process type.
    pub static parent_inactive: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(64.into(), CommonMetricData {
            name: "parent_inactive".into(),
            category: "power.wakeups_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.wakeups_per_thread.content_foreground
    ///
    /// How many times threads woke up and could have woken up a CPU core. Broken down
    /// by thread name for a given process type.
    pub static content_foreground: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(65.into(), CommonMetricData {
            name: "content_foreground".into(),
            category: "power.wakeups_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.wakeups_per_thread.content_background
    ///
    /// How many times threads woke up and could have woken up a CPU core. Broken down
    /// by thread name for a given process type.
    pub static content_background: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(66.into(), CommonMetricData {
            name: "content_background".into(),
            category: "power.wakeups_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.wakeups_per_thread.gpu_process
    ///
    /// How many times threads woke up and could have woken up a CPU core. Broken down
    /// by thread name for a given process type.
    pub static gpu_process: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(67.into(), CommonMetricData {
            name: "gpu_process".into(),
            category: "power.wakeups_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

}
pub mod power_cpu_ms_per_thread {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from power.cpu_ms_per_thread.parent_active
    ///
    /// How many miliseconds of CPU time were used. Broken down by thread name for a
    /// given process type.
    pub static parent_active: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(68.into(), CommonMetricData {
            name: "parent_active".into(),
            category: "power.cpu_ms_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.cpu_ms_per_thread.parent_inactive
    ///
    /// How many miliseconds of CPU time were used. Broken down by thread name for a
    /// given process type.
    pub static parent_inactive: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(69.into(), CommonMetricData {
            name: "parent_inactive".into(),
            category: "power.cpu_ms_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.cpu_ms_per_thread.content_foreground
    ///
    /// How many miliseconds of CPU time were used. Broken down by thread name for a
    /// given process type.
    pub static content_foreground: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(70.into(), CommonMetricData {
            name: "content_foreground".into(),
            category: "power.cpu_ms_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.cpu_ms_per_thread.content_background
    ///
    /// How many miliseconds of CPU time were used. Broken down by thread name for a
    /// given process type.
    pub static content_background: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(71.into(), CommonMetricData {
            name: "content_background".into(),
            category: "power.cpu_ms_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

    #[allow(non_upper_case_globals)]
    /// generated from power.cpu_ms_per_thread.gpu_process
    ///
    /// How many miliseconds of CPU time were used. Broken down by thread name for a
    /// given process type.
    pub static gpu_process: Lazy<LabeledMetric<LabeledCounterMetric>> = Lazy::new(|| {
        LabeledMetric::new(72.into(), CommonMetricData {
            name: "gpu_process".into(),
            category: "power.cpu_ms_per_thread".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["asynclogger".into(), "audioipc".into(), "audioipc_callback_rpc".into(), "audioipc_client_callback".into(), "audioipc_client_rpc".into(), "audioipc_devicecollection_rpc".into(), "audioipc_server_callback".into(), "audioipc_server_rpc".into(), "backgroundthreadpool".into(), "bgiothreadpool".into(), "bgreadurls".into(), "bhmgr_monitor".into(), "bhmgr_processor".into(), "cameras_ipc".into(), "capturethread".into(), "classifier_update".into(), "com_mta".into(), "compositor".into(), "cookie".into(), "cubeboperation".into(), "datachannel_io".into(), "dns_resolver".into(), "dom_worker".into(), "domcachethread".into(), "extensionprotocolhandler".into(), "font_loader".into(), "fontenumthread".into(), "fs_broker".into(), "geckomain".into(), "gmpthread".into(), "graphrunner".into(), "html5_parser".into(), "imagebridgechld".into(), "imageio".into(), "indexeddb".into(), "initfontlist".into(), "inotifyeventthread".into(), "ipc_i_o_child".into(), "ipc_i_o_parent".into(), "ipc_launch".into(), "ipdl_background".into(), "js_watchdog".into(), "jump_list".into(), "libwebrtcmodulethread".into(), "link_monitor".into(), "ls_thread".into(), "mediacache".into(), "mediadecoderstatemachine".into(), "mediapdecoder".into(), "mediasupervisor".into(), "mediatimer".into(), "mediatrackgrph".into(), "memorypoller".into(), "mozstorage".into(), "mtransport".into(), "netlink_monitor".into(), "pacerthread".into(), "permission".into(), "playeventsound".into(), "processhangmon".into(), "profilerchild".into(), "proxyresolution".into(), "quotamanager_io".into(), "registerfonts".into(), "remotebackbuffer".into(), "remotelzystream".into(), "remvidchild".into(), "renderer".into(), "sandboxreporter".into(), "savescripts".into(), "socket_thread".into(), "softwarevsyncthread".into(), "ssl_cert".into(), "startupcache".into(), "streamtrans".into(), "stylethread".into(), "swcomposite".into(), "taskcontroller".into(), "timer".into(), "toastbgthread".into(), "trr_background".into(), "untrusted_modules".into(), "url_classifier".into(), "videocapture".into(), "vsynciothread".into(), "webrtccallthread".into(), "webrtcworker".into(), "wincompositor".into(), "windowsvsyncthread".into(), "winwindowocclusioncalc".into(), "worker_launcher".into(), "wrrenderbackend".into(), "wrscenebuilder".into(), "wrscenebuilderlp".into(), "wrworker".into(), "wrworkerlp".into()]))
    });

}
pub mod power_battery {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from power.battery.percentage_when_user_active
    ///
    /// Records how many percent of battery was available for each period of user
    /// activity.
    pub static percentage_when_user_active: Lazy<CustomDistributionMetric> = Lazy::new(|| {
        CustomDistributionMetric::new(73.into(), CommonMetricData {
            name: "percentage_when_user_active".into(),
            category: "power.battery".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, 0, 100, 100, HistogramType::Linear)
    });

}
pub mod search_engine_default {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.default.engine_id
    ///
    /// The telemetry id of the search engine.
    /// For application provided engines, this is either supplied by the
    /// configuration or from the first part of the associated WebExtension Id.
    /// For other engines, this is `other-<extensionName>`.
    pub static engine_id: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(74.into(), CommonMetricData {
            name: "engine_id".into(),
            category: "search.engine.default".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.default.display_name
    ///
    /// The display name of the user's default engine.
    pub static display_name: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(75.into(), CommonMetricData {
            name: "display_name".into(),
            category: "search.engine.default".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.default.load_path
    ///
    /// A path relating to where the search engine was installed/loaded from.
    /// For example:
    ///   `[other]addEngineWithDetails:<extension id>` for a WebExtension based
    ///   engine.
    ///   `[https]developer.mozilla.org/mdn-web-docs.xml` for an OpenSearch based
    ///   engine.
    /// Note: this metric is truncated at 100 characters.
    pub static load_path: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(76.into(), CommonMetricData {
            name: "load_path".into(),
            category: "search.engine.default".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.default.submission_url
    ///
    /// The submission URL of the default engine. This is only reported in the
    /// cases where:
    ///   The engine is an application provided engine.
    ///   The engine has the same name as an application provided engine.
    ///   The engine matches one of a specific list of well known search engines.
    pub static submission_url: Lazy<UrlMetric> = Lazy::new(|| {
        UrlMetric::new(77.into(), CommonMetricData {
            name: "submission_url".into(),
            category: "search.engine.default".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.default.verified
    ///
    /// The verified status of the search engine.
    /// For application provided engines, this will always be `default`.
    /// For other engines this will either be `verified` or `unverified` depending
    /// on if the loadPathHash is valid.
    pub static verified: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(78.into(), CommonMetricData {
            name: "verified".into(),
            category: "search.engine.default".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod search_engine_private {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.private.engine_id
    ///
    /// The telemetry id of the search engine.
    /// For application provided engines, this is either supplied by the
    /// configuration or from the first part of the associated WebExtension Id.
    /// For other engines, this is `other-<extensionName>`.
    /// If this string is an empty string (`""`), this means that one or both of
    /// the preferences `browser.search.separatePrivateDefault` and
    /// `browser.search.separatePrivateDefault.ui.enabled` are set to false.
    /// It is possible that the user selects the same private engine as for the
    /// default engine, and hence both versions of these fields will be filled in.
    pub static engine_id: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(79.into(), CommonMetricData {
            name: "engine_id".into(),
            category: "search.engine.private".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.private.display_name
    ///
    /// The display name of the user's default engine.
    /// If this string is an empty string (`""`), this means that one or both of
    /// the preferences `browser.search.separatePrivateDefault` and
    /// `browser.search.separatePrivateDefault.ui.enabled` are set to false.
    /// It is possible that the user selects the same private engine as for the
    /// default engine, and hence both versions of these fields will be filled in.
    pub static display_name: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(80.into(), CommonMetricData {
            name: "display_name".into(),
            category: "search.engine.private".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.private.load_path
    ///
    /// A path relating to where the search engine was installed/loaded from.
    /// For example:
    ///   `[other]addEngineWithDetails:<extension id>` for a WebExtension based
    ///   engine.
    ///   `[https]developer.mozilla.org/mdn-web-docs.xml` for an OpenSearch based
    ///   engine.
    /// Note: this metric is truncated at 100 characters.
    /// If this string is an empty string (`""`), this means that one or both of
    /// the preferences `browser.search.separatePrivateDefault` and
    /// `browser.search.separatePrivateDefault.ui.enabled` are set to false.
    /// It is possible that the user selects the same private engine as for the
    /// default engine, and hence both versions of these fields will be filled in.
    pub static load_path: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(81.into(), CommonMetricData {
            name: "load_path".into(),
            category: "search.engine.private".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.private.submission_url
    ///
    /// The submission URL of the default engine. This is only reported in the
    /// cases where:
    ///   The engine is an application provided engine.
    ///   The engine has the same name as an application provided engine.
    ///   The engine matches one of a specific list of well known search engines.
    /// If this string is an empty string (`""`), this means that one or both of
    /// the preferences `browser.search.separatePrivateDefault` and
    /// `browser.search.separatePrivateDefault.ui.enabled` are set to false.
    /// It is possible that the user selects the same private engine as for the
    /// default engine, and hence both versions of these fields will be filled in.
    pub static submission_url: Lazy<UrlMetric> = Lazy::new(|| {
        UrlMetric::new(82.into(), CommonMetricData {
            name: "submission_url".into(),
            category: "search.engine.private".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from search.engine.private.verified
    ///
    /// The verified status of the search engine.
    /// For application provided engines, this will always be `default`.
    /// For other engines this will either be `verified` or `unverified` depending
    /// on if the loadPathHash is valid.
    /// If this string is an empty string (`""`), this means that one or both of
    /// the preferences `browser.search.separatePrivateDefault` and
    /// `browser.search.separatePrivateDefault.ui.enabled` are set to false.
    /// It is possible that the user selects the same private engine as for the
    /// default engine, and hence both versions of these fields will be filled in.
    pub static verified: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(83.into(), CommonMetricData {
            name: "verified".into(),
            category: "search.engine.private".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod legacy_telemetry {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from legacy.telemetry.client_id
    ///
    /// The client_id according to Telemetry.
    /// Might not always have a value due to being too early for it to have
    /// loaded.
    /// Value may be the canary client id `c0ffeec0-ffee-c0ff-eec0-ffeec0ffeec0`
    /// in pings near when the data upload pref is disabled (if Telemetry gets
    /// to go first), or between when a client_id has been removed and when it
    /// has been regenerated.
    /// Does not need to be sent in the Glean "deletion-request" ping.
    pub static client_id: Lazy<UuidMetric> = Lazy::new(|| {
        UuidMetric::new(84.into(), CommonMetricData {
            name: "client_id".into(),
            category: "legacy.telemetry".into(),
            send_in_pings: vec!["events".into(), "metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod background_update {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from background_update.client_id
    ///
    /// The Telemetry client ID of the default profile.
    pub static client_id: Lazy<UuidMetric> = Lazy::new(|| {
        UuidMetric::new(85.into(), CommonMetricData {
            name: "client_id".into(),
            category: "background_update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from background_update.final_state
    ///
    /// String description of the final state the update state machine reached.
    pub static final_state: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(86.into(), CommonMetricData {
            name: "final_state".into(),
            category: "background_update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from background_update.states
    ///
    /// Ordered list of string descriptions of the states that the update state machine
    /// reached.
    pub static states: Lazy<StringListMetric> = Lazy::new(|| {
        StringListMetric::new(87.into(), CommonMetricData {
            name: "states".into(),
            category: "background_update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from background_update.reasons
    ///
    /// List of reasons that the background update task did not run.
    pub static reasons: Lazy<StringListMetric> = Lazy::new(|| {
        StringListMetric::new(88.into(), CommonMetricData {
            name: "reasons".into(),
            category: "background_update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from background_update.exit_code_success
    ///
    /// True if the exit code/status of the background update task is 0, which means
    /// success.
    pub static exit_code_success: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(89.into(), CommonMetricData {
            name: "exit_code_success".into(),
            category: "background_update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from background_update.exit_code_exception
    ///
    /// True if the exit code/status of the background update task is 3, which means an
    /// exception was thrown.
    pub static exit_code_exception: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(90.into(), CommonMetricData {
            name: "exit_code_exception".into(),
            category: "background_update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod update {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from update.service_enabled
    ///
    /// Preference "app.update.service.enabled": whether the Mozilla Maintenance
    /// Service is enabled.
    pub static service_enabled: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(91.into(), CommonMetricData {
            name: "service_enabled".into(),
            category: "update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from update.auto_download
    ///
    /// Per-installation preference "app.update.auto": whether to fetch and install
    /// updates without user intervention.
    pub static auto_download: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(92.into(), CommonMetricData {
            name: "auto_download".into(),
            category: "update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from update.background_update
    ///
    /// Per-installation preference "app.update.background.enabled": whether to fetch
    /// and install updates in the background when Firefox is not running.
    pub static background_update: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(93.into(), CommonMetricData {
            name: "background_update".into(),
            category: "update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from update.enabled
    ///
    /// True when policies are disabled or when the "DisableAppUpdate" is not in
    /// effect.
    pub static enabled: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(94.into(), CommonMetricData {
            name: "enabled".into(),
            category: "update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from update.channel
    ///
    /// The update channel.
    pub static channel: Lazy<StringMetric> = Lazy::new(|| {
        StringMetric::new(95.into(), CommonMetricData {
            name: "channel".into(),
            category: "update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from update.can_usually_apply_updates
    ///
    /// Whether or not the Update Service can usually download and install updates. See
    /// `canUsuallyApplyUpdates` in https://searchfox.org/mozilla-
    /// central/source/toolkit/mozapps/update/nsIUpdateService.idl.
    pub static can_usually_apply_updates: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(96.into(), CommonMetricData {
            name: "can_usually_apply_updates".into(),
            category: "update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from update.can_usually_check_for_updates
    ///
    /// Whether or not the Update Service can usually check for updates. See
    /// `canUsuallyCheckForUpdates` in https://searchfox.org/mozilla-
    /// central/source/toolkit/mozapps/update/nsIUpdateService.idl.
    pub static can_usually_check_for_updates: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(97.into(), CommonMetricData {
            name: "can_usually_check_for_updates".into(),
            category: "update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from update.can_usually_stage_updates
    ///
    /// Whether the Update Service is usually able to stage updates. See
    /// `canUsuallyStageUpdates` in https://searchfox.org/mozilla-
    /// central/source/toolkit/mozapps/update/nsIUpdateService.idl.
    pub static can_usually_stage_updates: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(98.into(), CommonMetricData {
            name: "can_usually_stage_updates".into(),
            category: "update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

    #[allow(non_upper_case_globals)]
    /// generated from update.can_usually_use_bits
    ///
    /// On Windows, whether the Update Service can usually use BITS. See
    /// `canUsuallyUseBits` in https://searchfox.org/mozilla-
    /// central/source/toolkit/mozapps/update/nsIUpdateService.idl.
    pub static can_usually_use_bits: Lazy<BooleanMetric> = Lazy::new(|| {
        BooleanMetric::new(99.into(), CommonMetricData {
            name: "can_usually_use_bits".into(),
            category: "update".into(),
            send_in_pings: vec!["background-update".into()],
            lifetime: Lifetime::Application,
            disabled: false,
            ..Default::default()
        })
    });

}
pub mod startup {
    use crate::private::*;
    use glean::CommonMetricData;
    #[allow(unused_imports)] // HistogramType might be unusued, let's avoid warnings
    use glean::HistogramType;
    use once_cell::sync::Lazy;

    #[allow(non_upper_case_globals)]
    /// generated from startup.run_from_dmg_install_outcome
    ///
    /// The outcome after the app detected that it was running from DMG or an app
    /// translocated location and should offer to install and relaunch itself.
    pub static run_from_dmg_install_outcome: Lazy<LabeledMetric<LabeledBooleanMetric>> = Lazy::new(|| {
        LabeledMetric::new(100.into(), CommonMetricData {
            name: "run_from_dmg_install_outcome".into(),
            category: "startup".into(),
            send_in_pings: vec!["metrics".into()],
            lifetime: Lifetime::Ping,
            disabled: false,
            ..Default::default()
        }, Some(vec!["installed_and_relaunched".into(), "non_privileged_install_failed".into(), "privileged_install_failed".into(), "root_applications_dir_missing".into(), "user_accepted_launch_existing".into(), "user_declined_install_prompt".into(), "user_declined_launch_existing".into()]))
    });

}

#[allow(dead_code)]
pub(crate) mod __glean_metric_maps {
    use std::collections::HashMap;

    use super::{id_for_extra_key, extra_keys_len};
    use crate::private::*;
    use once_cell::sync::Lazy;

    pub static COUNTER_MAP: Lazy<HashMap<MetricId, &Lazy<CounterMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(12);
        map.insert(1.into(), &super::browser_engagement::active_ticks);
        map.insert(2.into(), &super::browser_engagement::uri_count);
        map.insert(13.into(), &super::fog_ipc::replay_failures);
        map.insert(16.into(), &super::fog_ipc::flush_failures);
        map.insert(17.into(), &super::fog_ipc::shutdown_registration_failures);
        map.insert(18.into(), &super::test_only::bad_code);
        map.insert(36.into(), &super::test_only_ipc::a_counter);
        map.insert(55.into(), &super::power::cpu_time_bogus_values);
        map.insert(58.into(), &super::power::gpu_time_bogus_values);
        map.insert(60.into(), &super::power::total_cpu_time_ms);
        map.insert(61.into(), &super::power::total_gpu_time_ms);
        map.insert(62.into(), &super::power::total_thread_wakeups);
        map
    });

    pub static LABELED_COUNTER_MAP: Lazy<HashMap<MetricId, &Lazy<LabeledMetric<LabeledCounterMetric>>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(18);
        map.insert(3.into(), &super::gmp::update_xml_fetch_result);
        map.insert(27.into(), &super::test_only::mabels_kitchen_counters);
        map.insert(28.into(), &super::test_only::mabels_bathroom_counters);
        map.insert(48.into(), &super::test_only_ipc::a_labeled_counter);
        map.insert(49.into(), &super::test_only_ipc::another_labeled_counter);
        map.insert(56.into(), &super::power::cpu_time_per_process_type_ms);
        map.insert(57.into(), &super::power::gpu_time_per_process_type_ms);
        map.insert(59.into(), &super::power::wakeups_per_process_type);
        map.insert(63.into(), &super::power_wakeups_per_thread::parent_active);
        map.insert(64.into(), &super::power_wakeups_per_thread::parent_inactive);
        map.insert(65.into(), &super::power_wakeups_per_thread::content_foreground);
        map.insert(66.into(), &super::power_wakeups_per_thread::content_background);
        map.insert(67.into(), &super::power_wakeups_per_thread::gpu_process);
        map.insert(68.into(), &super::power_cpu_ms_per_thread::parent_active);
        map.insert(69.into(), &super::power_cpu_ms_per_thread::parent_inactive);
        map.insert(70.into(), &super::power_cpu_ms_per_thread::content_foreground);
        map.insert(71.into(), &super::power_cpu_ms_per_thread::content_background);
        map.insert(72.into(), &super::power_cpu_ms_per_thread::gpu_process);
        map
    });

    pub static TIMING_DISTRIBUTION_MAP: Lazy<HashMap<MetricId, &Lazy<TimingDistributionMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(8);
        map.insert(5.into(), &super::paint::build_displaylist_time);
        map.insert(6.into(), &super::wr::rasterize_glyphs_time);
        map.insert(7.into(), &super::wr::framebuild_time);
        map.insert(8.into(), &super::wr::scenebuild_time);
        map.insert(9.into(), &super::wr::sceneswap_time);
        map.insert(15.into(), &super::fog_ipc::flush_durations);
        map.insert(26.into(), &super::test_only::what_time_is_it);
        map.insert(41.into(), &super::test_only_ipc::a_timing_dist);
        map
    });

    pub static TIMESPAN_MAP: Lazy<HashMap<MetricId, &Lazy<TimespanMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(5);
        map.insert(10.into(), &super::extensions::startup_cache_load_time);
        map.insert(11.into(), &super::fog::initialization);
        map.insert(19.into(), &super::test_only::can_we_time_it);
        map.insert(31.into(), &super::test_only::mirror_time);
        map.insert(32.into(), &super::test_only::mirror_time_nanos);
        map
    });

    pub static BOOLEAN_MAP: Lazy<HashMap<MetricId, &Lazy<BooleanMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(14);
        map.insert(12.into(), &super::fog::failed_idle_registration);
        map.insert(24.into(), &super::test_only::can_we_flag_it);
        map.insert(34.into(), &super::test_only::one_ping_one_bool);
        map.insert(37.into(), &super::test_only_ipc::a_bool);
        map.insert(89.into(), &super::background_update::exit_code_success);
        map.insert(90.into(), &super::background_update::exit_code_exception);
        map.insert(91.into(), &super::update::service_enabled);
        map.insert(92.into(), &super::update::auto_download);
        map.insert(93.into(), &super::update::background_update);
        map.insert(94.into(), &super::update::enabled);
        map.insert(96.into(), &super::update::can_usually_apply_updates);
        map.insert(97.into(), &super::update::can_usually_check_for_updates);
        map.insert(98.into(), &super::update::can_usually_stage_updates);
        map.insert(99.into(), &super::update::can_usually_use_bits);
        map
    });

    pub static MEMORY_DISTRIBUTION_MAP: Lazy<HashMap<MetricId, &Lazy<MemoryDistributionMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(3);
        map.insert(14.into(), &super::fog_ipc::buffer_sizes);
        map.insert(25.into(), &super::test_only::do_you_remember);
        map.insert(40.into(), &super::test_only_ipc::a_memory_dist);
        map
    });

    pub static STRING_MAP: Lazy<HashMap<MetricId, &Lazy<StringMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(12);
        map.insert(20.into(), &super::test_only::cheesy_string);
        map.insert(39.into(), &super::test_only_ipc::a_string);
        map.insert(74.into(), &super::search_engine_default::engine_id);
        map.insert(75.into(), &super::search_engine_default::display_name);
        map.insert(76.into(), &super::search_engine_default::load_path);
        map.insert(78.into(), &super::search_engine_default::verified);
        map.insert(79.into(), &super::search_engine_private::engine_id);
        map.insert(80.into(), &super::search_engine_private::display_name);
        map.insert(81.into(), &super::search_engine_private::load_path);
        map.insert(83.into(), &super::search_engine_private::verified);
        map.insert(86.into(), &super::background_update::final_state);
        map.insert(95.into(), &super::update::channel);
        map
    });

    pub static STRING_LIST_MAP: Lazy<HashMap<MetricId, &Lazy<StringListMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(4);
        map.insert(21.into(), &super::test_only::cheesy_string_list);
        map.insert(43.into(), &super::test_only_ipc::a_string_list);
        map.insert(87.into(), &super::background_update::states);
        map.insert(88.into(), &super::background_update::reasons);
        map
    });

    pub static DATETIME_MAP: Lazy<HashMap<MetricId, &Lazy<DatetimeMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(2);
        map.insert(22.into(), &super::test_only::what_a_date);
        map.insert(38.into(), &super::test_only_ipc::a_date);
        map
    });

    pub static UUID_MAP: Lazy<HashMap<MetricId, &Lazy<UuidMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(4);
        map.insert(23.into(), &super::test_only::what_id_it);
        map.insert(47.into(), &super::test_only_ipc::a_uuid);
        map.insert(84.into(), &super::legacy_telemetry::client_id);
        map.insert(85.into(), &super::background_update::client_id);
        map
    });

    pub static LABELED_BOOLEAN_MAP: Lazy<HashMap<MetricId, &Lazy<LabeledMetric<LabeledBooleanMetric>>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(3);
        map.insert(29.into(), &super::test_only::mabels_like_balloons);
        map.insert(33.into(), &super::test_only::mirrors_for_labeled_bools);
        map.insert(100.into(), &super::startup::run_from_dmg_install_outcome);
        map
    });

    pub static LABELED_STRING_MAP: Lazy<HashMap<MetricId, &Lazy<LabeledMetric<LabeledStringMetric>>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(1);
        map.insert(30.into(), &super::test_only::mabels_balloon_strings);
        map
    });

    pub static QUANTITY_MAP: Lazy<HashMap<MetricId, &Lazy<QuantityMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(2);
        map.insert(35.into(), &super::test_only::meaning_of_life);
        map.insert(50.into(), &super::test_only_ipc::a_quantity);
        map
    });

    pub static CUSTOM_DISTRIBUTION_MAP: Lazy<HashMap<MetricId, &Lazy<CustomDistributionMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(2);
        map.insert(42.into(), &super::test_only_ipc::a_custom_dist);
        map.insert(73.into(), &super::power_battery::percentage_when_user_active);
        map
    });

    pub static RATE_MAP: Lazy<HashMap<MetricId, &Lazy<RateMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(1);
        map.insert(51.into(), &super::test_only_ipc::irate);
        map
    });

    pub static NUMERATOR_MAP: Lazy<HashMap<MetricId, &Lazy<NumeratorMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(1);
        map.insert(52.into(), &super::test_only_ipc::rate_with_external_denominator);
        map
    });

    pub static DENOMINATOR_MAP: Lazy<HashMap<MetricId, &Lazy<DenominatorMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(1);
        map.insert(53.into(), &super::test_only_ipc::an_external_denominator);
        map
    });

    pub static URL_MAP: Lazy<HashMap<MetricId, &Lazy<UrlMetric>>> = Lazy::new(|| {
        let mut map = HashMap::with_capacity(3);
        map.insert(54.into(), &super::test_only_ipc::a_url);
        map.insert(77.into(), &super::search_engine_default::submission_url);
        map.insert(82.into(), &super::search_engine_private::submission_url);
        map
    });


    /// Wrapper to record an event based on its metric ID.
    ///
    /// # Arguments
    ///
    /// * `metric_id` - The metric's ID to look up
    /// * `extra`     - An map of (extra key id, string) pairs.
    ///                 The map will be decoded into the appropriate `ExtraKeys` type.
    /// # Returns
    ///
    /// Returns `Ok(())` if the event was found and `record` was called with the given `extra`,
    /// or an `EventRecordingError::InvalidId` if no event by that ID exists
    /// or an `EventRecordingError::InvalidExtraKey` if the `extra` map could not be deserialized.
    pub(crate) fn record_event_by_id(metric_id: u32, extra: HashMap<i32, String>) -> Result<(), EventRecordingError> {
        match metric_id {
          4 => {
              assert!(
                  extra_keys_len(&super::perf::page_load) != 0 || extra.is_empty(),
                  "No extra keys allowed, but some were passed"
              );

              super::perf::page_load.record_raw(extra);
              Ok(())
          }
          44 => {
              assert!(
                  extra_keys_len(&super::test_only_ipc::an_event) != 0 || extra.is_empty(),
                  "No extra keys allowed, but some were passed"
              );

              super::test_only_ipc::an_event.record_raw(extra);
              Ok(())
          }
          45 => {
              assert!(
                  extra_keys_len(&super::test_only_ipc::event_with_extra) != 0 || extra.is_empty(),
                  "No extra keys allowed, but some were passed"
              );

              super::test_only_ipc::event_with_extra.record_raw(extra);
              Ok(())
          }
          46 => {
              assert!(
                  extra_keys_len(&super::test_only_ipc::no_extra_event) != 0 || extra.is_empty(),
                  "No extra keys allowed, but some were passed"
              );

              super::test_only_ipc::no_extra_event.record_raw(extra);
              Ok(())
          }
           _ => Err(EventRecordingError::InvalidId),
        }
    }

    /// Wrapper to record an event based on its metric ID, with a provided timestamp.
    ///
    /// # Arguments
    ///
    /// * `metric_id` - The metric's ID to look up
    /// * `timestamp` - The time at which this event was recorded.
    /// * `extra`     - An map of (extra key id, string) pairs.
    ///                 The map will be decoded into the appropriate `ExtraKeys` type.
    /// # Returns
    ///
    /// Returns `Ok(())` if the event was found and `record` was called with the given `extra`,
    /// or an `EventRecordingError::InvalidId` if no event by that ID exists
    /// or an `EventRecordingError::InvalidExtraKey` if the event doesn't take extra pairs,
    /// but some are passed in.
    pub(crate) fn record_event_by_id_with_time(metric_id: MetricId, timestamp: u64, extra: HashMap<i32, String>) -> Result<(), EventRecordingError> {
        match metric_id {
          MetricId(4) => {
              if extra_keys_len(&super::perf::page_load) == 0 && !extra.is_empty() {
                return Err(EventRecordingError::InvalidExtraKey);
              }

              super::perf::page_load.record_with_time(timestamp, extra);
              Ok(())
          }
          MetricId(44) => {
              if extra_keys_len(&super::test_only_ipc::an_event) == 0 && !extra.is_empty() {
                return Err(EventRecordingError::InvalidExtraKey);
              }

              super::test_only_ipc::an_event.record_with_time(timestamp, extra);
              Ok(())
          }
          MetricId(45) => {
              if extra_keys_len(&super::test_only_ipc::event_with_extra) == 0 && !extra.is_empty() {
                return Err(EventRecordingError::InvalidExtraKey);
              }

              super::test_only_ipc::event_with_extra.record_with_time(timestamp, extra);
              Ok(())
          }
          MetricId(46) => {
              if extra_keys_len(&super::test_only_ipc::no_extra_event) == 0 && !extra.is_empty() {
                return Err(EventRecordingError::InvalidExtraKey);
              }

              super::test_only_ipc::no_extra_event.record_with_time(timestamp, extra);
              Ok(())
          }
           _ => Err(EventRecordingError::InvalidId),
        }
    }

    /// Wrapper to record an event based on its metric ID.
    ///
    /// # Arguments
    ///
    /// * `metric_id` - The metric's ID to look up
    /// * `extra`     - An map of (string, string) pairs.
    ///                 The map will be decoded into the appropriate `ExtraKeys` types.
    /// # Returns
    ///
    /// Returns `Ok(())` if the event was found and `record` was called with the given `extra`,
    /// or an `EventRecordingError::InvalidId` if no event by that ID exists
    /// or an `EventRecordingError::InvalidExtraKey` if the `extra` map could not be deserialized.
    pub(crate) fn record_event_by_id_with_strings(metric_id: u32, extra: HashMap<String, String>) -> Result<(), EventRecordingError> {
        match metric_id {
          4 => {
              assert!(
                  extra_keys_len(&super::perf::page_load) != 0 || extra.is_empty(),
                  "No extra keys allowed, but some were passed"
              );

              let extra = extra
                  .into_iter()
                  .map(|(k, v)| id_for_extra_key(&*k, &super::perf::page_load).map(|k| (k, v)))
                  .collect::<Result<HashMap<_, _>, _>>()?;
              super::perf::page_load.record_raw(extra);
              Ok(())
          }
          44 => {
              assert!(
                  extra_keys_len(&super::test_only_ipc::an_event) != 0 || extra.is_empty(),
                  "No extra keys allowed, but some were passed"
              );

              let extra = extra
                  .into_iter()
                  .map(|(k, v)| id_for_extra_key(&*k, &super::test_only_ipc::an_event).map(|k| (k, v)))
                  .collect::<Result<HashMap<_, _>, _>>()?;
              super::test_only_ipc::an_event.record_raw(extra);
              Ok(())
          }
          45 => {
              assert!(
                  extra_keys_len(&super::test_only_ipc::event_with_extra) != 0 || extra.is_empty(),
                  "No extra keys allowed, but some were passed"
              );

              let extra = extra
                  .into_iter()
                  .map(|(k, v)| id_for_extra_key(&*k, &super::test_only_ipc::event_with_extra).map(|k| (k, v)))
                  .collect::<Result<HashMap<_, _>, _>>()?;
              super::test_only_ipc::event_with_extra.record_raw(extra);
              Ok(())
          }
          46 => {
              assert!(
                  extra_keys_len(&super::test_only_ipc::no_extra_event) != 0 || extra.is_empty(),
                  "No extra keys allowed, but some were passed"
              );

              let extra = extra
                  .into_iter()
                  .map(|(k, v)| id_for_extra_key(&*k, &super::test_only_ipc::no_extra_event).map(|k| (k, v)))
                  .collect::<Result<HashMap<_, _>, _>>()?;
              super::test_only_ipc::no_extra_event.record_raw(extra);
              Ok(())
          }
           _ => Err(EventRecordingError::InvalidId),
        }
    }

    /// Wrapper to get the currently stored events for event metric.
    ///
    /// # Arguments
    ///
    /// * `metric_id` - The metric's ID to look up
    /// * `ping_name` - (Optional) The ping name to look into.
    ///                 Defaults to the first value in `send_in_pings`.
    ///
    /// # Returns
    ///
    /// Returns the recorded events or `None` if nothing stored.
    ///
    /// # Panics
    ///
    /// Panics if no event by the given metric ID could be found.
    pub(crate) fn event_test_get_value_wrapper(metric_id: u32, ping_name: Option<String>) -> Option<Vec<RecordedEvent>> {
        match metric_id {
           4 => super::perf::page_load.test_get_value(ping_name.as_deref()),
           44 => super::test_only_ipc::an_event.test_get_value(ping_name.as_deref()),
           45 => super::test_only_ipc::event_with_extra.test_get_value(ping_name.as_deref()),
           46 => super::test_only_ipc::no_extra_event.test_get_value(ping_name.as_deref()),
           _ => panic!("No event for metric id {}", metric_id),
        }
    }

    /// Check the provided event for errors.
    ///
    /// # Arguments
    ///
    /// * `metric_id` - The metric's ID to look up
    /// * `ping_name` - (Optional) The ping name to look into.
    ///                 Defaults to the first value in `send_in_pings`.
    ///
    /// # Returns
    ///
    /// Returns a string for the recorded error or `None`.
    ///
    /// # Panics
    ///
    /// Panics if no event by the given metric ID could be found.
    #[allow(unused_variables)]
    pub(crate) fn event_test_get_error(metric_id: u32, ping_name: Option<String>) -> Option<String> {
        #[cfg(feature = "with_gecko")]
        match metric_id {
           4 => test_get_errors_string!(super::perf::page_load, ping_name),
           44 => test_get_errors_string!(super::test_only_ipc::an_event, ping_name),
           45 => test_get_errors_string!(super::test_only_ipc::event_with_extra, ping_name),
           46 => test_get_errors_string!(super::test_only_ipc::no_extra_event, ping_name),
           _ => panic!("No event for metric id {}", metric_id),
        }

        #[cfg(not(feature = "with_gecko"))]
        {
            return None;
        }
    }

    pub(crate) mod submetric_maps {
        use std::sync::{
          atomic::AtomicU32,
          RwLock,
        };
        use super::*;

        pub(crate) const MIN_LABELED_SUBMETRIC_ID: u32 = 134217729;
        pub(crate) static NEXT_LABELED_SUBMETRIC_ID: AtomicU32 = AtomicU32::new(MIN_LABELED_SUBMETRIC_ID);
        pub(crate) static LABELED_METRICS_TO_IDS: Lazy<RwLock<HashMap<(u32, String), u32>>> = Lazy::new(||
            RwLock::new(HashMap::new())
        );

        pub static COUNTER_MAP: Lazy<RwLock<HashMap<MetricId, LabeledCounterMetric>>> = Lazy::new(||
            RwLock::new(HashMap::new())
        );
        pub static BOOLEAN_MAP: Lazy<RwLock<HashMap<MetricId, LabeledBooleanMetric>>> = Lazy::new(||
            RwLock::new(HashMap::new())
        );
        pub static STRING_MAP: Lazy<RwLock<HashMap<MetricId, LabeledStringMetric>>> = Lazy::new(||
            RwLock::new(HashMap::new())
        );
    }
}

